#include "Audioh.h"
/*

Example usage:

Audioh *a=new Audioh();
int buf=a->add("file.ogg");
int sid=a->sid_new(buf);
a->sid_play(sid);

*/

// Constructor //
// =========== //
Audioh::Audioh(bool verbose)
{
	// Initialize buffers
	this->n_buf=0;
	this->buf = (ALuint *) malloc(1);

	// Initialize sources
	this->n_sid=0;
	this->sid = (ALuint *) malloc(1);

	this->name = (char **) malloc(1);

	this->verbose=verbose;

	if (SOUND_ENABLED)
		readConfig();

#if OPENAL
	float zeroes[3] = { 0.0f,0.0f,0.0f };
	// Initializing audio device
	this->dev=alcOpenDevice(NULL);
	if (!this->dev) 
	{
		printf("[E] Audio device yet in use.\n");
		throw 1;
	}
	
	// Initializing audio context
	this->cid=alcCreateContext(this->dev,NULL);
	if (!this->cid)
	{
		alcCloseDevice(dev);
		SOUND_ENABLED=0;
	}
	alcMakeContextCurrent(this->cid);

	// Set default listener velocity
	alListenerfv(AL_VELOCITY,zeroes);

	printf("[>] Audio driver: OpenAL\n");
#endif

#if SDLSOUND
	int err;

	SDL_Init(SDL_INIT_AUDIO);

	err=Mix_OpenAudio(MIX_DEFAULT_FREQUENCY,MIX_DEFAULT_FORMAT,1,4096);
	Mix_AllocateChannels(50); // brbrb
	
	if (err)
	{
		printf("[E] Cannot initialize SDL_sound.\n");
		SOUND_ENABLED=0;
	}

	printf("[>] Audio driver: SDL\n");
#endif
}

// Destructor //
// ========== //

Audioh::~Audioh()
{
	// Free Buffers.
	if (this->buf)
	free( this->buf );

	// Free Sources
	if (this->sid)
	free (this->sid);

	for (int i=0;i<n_buf;i++)
	if (this->name) free(this->name[i]);
	free(this->name);

#if OPENAL
	// Free Device.
	if (this->dev)
	alcCloseDevice(this->dev);
#endif
#if SDLSOUND
	Mix_CloseAudio();
#endif

	printf("[<] Audioh class destroyed.\n");
}

// Configuration
void Audioh::readConfig()
{
	FILE *fd;
	char buf[128];
	char name[128];
	char *args;

	if (SOUND_ENABLED==false)
	return;

	chCfgDir();

	fd=fopen("config.snh","r");
	if (!fd) 
	{
		printf("config.snh not found...loading defaults.\n");
		SOUND_ENABLED=1;
		SOUND_VOLUME=100;
	} else {
		while(!feof(fd))
		{
			if (fgets(buf,100,fd)==NULL) continue;
			
			sscanf(buf,"%s",name);
			if (name[0]=='#') continue;

			args=strstr(buf,name)+strlen(name);

			if (!strcmp("ENABLE",name)) 
				sscanf(args,"%d",&SOUND_ENABLED);
			else if (!strcmp("VOLUME",name)) 
				sscanf(args,"%d",&SOUND_VOLUME);
		}
		fclose(fd);
	}
}


// Add new sound file //
// ================== //

int
Audioh::add(char *f)
{
	this->n_buf=0;

	if (!SOUND_ENABLED)
		return 0;

	if (verbose)
	printf("audioh: add(%s)\n",f);

#if OPENAL
	ALboolean err;
	ALsizei size,bits,freq;
	ALint format;
	void *wave;

	// Load Wav file

	err=alutLoadWAV(f,&wave,&format,&size,&bits,&freq);

	if (err==AL_FALSE) 
	{
		printf("   [-] - audioh - invalid file \"%s\".\n",f);
		return SID_NULL;
	}

	this->n_buf++;
	this->buf = (ALuint *) realloc (this->buf, this->n_buf*sizeof(ALuint));
	if (!this->buf) throw 2;

	this->name = (char **) realloc(this->name,this->n_buf*sizeof(char));
	this->name[n_buf-1] = strdup(f);

	alGenBuffers(1,&this->buf[this->n_buf-1]);

	alBufferData(this->buf[this->n_buf-1],
			format,wave,size,freq);

	printf("   [+] - audioh - file \"%s\" loaded OK.\n",f);
	if (verbose)
	printf("       - format: %d | freq %d | bits %d | size %d\n",
			format,freq,bits,size );

	free ( wave );

	return (this->n_buf-1);
#endif

#if SDLSOUND
	Mix_Chunk *chu;

	chu=Mix_LoadWAV(f);

	if (chu==NULL)
	{
		printf("[-] %s : %s\n",f,Mix_GetError());
		return SID_NULL;
	}

	this->n_buf++;
	this->buf=(ALuint *)realloc(this->buf,this->n_buf*sizeof(int));
	if (!this->buf) throw 2;
	this->buf[this->n_buf-1]=(ALuint)chu;

	this->name = (char **) realloc(this->name,this->n_buf*sizeof(char));
	this->name[n_buf-1] = strdup(f);


	return (this->n_buf-1);
#endif

	return SID_NULL;
}

// Set listener position //
// ===================== //

void
Audioh::setListener(float x,float y,float z)
{
	this->setListener(x,y,z,x,y,z);
}

void
Audioh::setListener(float x,float y,float z, float x1,float y1,float z1)
{
#if OPENAL
	float pos[6]={x,y,z,x1,y1,z1};

	alListenerfv (AL_POSITION,pos);
	alListenerfv (AL_DIRECTION,pos);
#endif
}

// Volume Manager //
// ============== //

void
Audioh::setVolume(int v)
{
	float vol;

	// Clean input.
	if (v>100) v=100; else
	if (v<0)   v=0;

	if (!SOUND_ENABLED)
		return;

	if (verbose)
	printf("audioh: setVolume(%d)\n",v);

#if OPENAL
	vol=(float)v/100.0f;
	alListenerf ( AL_GAIN,vol ); // Set new volume
#endif

#if SDLSOUND
	vol=v;
	Mix_VolumeMusic( (int) (vol/100)*128 );
#endif
}

// Play buffer on new source //
// ========================= //
//private:
bool
Audioh::checkSid(ALuint sid)
{
	if (sid==SID_NULL)
		return false;
	if (sid > this->n_sid) // invalid pointer
		return false;
	else if (!this->sid[sid]) // destroyed SID
		return false;
	else	return true;
}

//public:
int
Audioh::sid_new(ALuint buf)
{
	unsigned int i=SID_NULL;

#if OPENAL || SDLSOUND
	if (verbose)
	printf("audioh: sid_new(%d)\n",buf);

	for(i=0; i < this->n_sid; i++)
		if (!this->sid[i]) break;

	if (i==this->n_sid) // Must realloc
	{
		this->n_sid++;
		this->sid = (ALuint *) 
			realloc(this->sid,this->n_sid*sizeof(ALuint));
	}
#endif

#if OPENAL
	alGenSources(1, &this->sid[i]);
	alSource3f ( this->sid[i], AL_VELOCITY, 0.0f,0.0f,0.0f );
	alSourcei  ( this->sid[i], AL_BUFFER, this->buf[buf] );
#endif

#if SDLSOUND
	Mix_Chunk *chu=(Mix_Chunk*)this->buf[buf];
	if (chu==NULL)
	{
		i=SID_NULL;
		this->sid[i]=SID_NULL;
	} else {
		this->sid[i]=SID_ISOK;

		/* assign new channel */
		Mix_PlayChannel(i,chu,0);
		Mix_Pause(i);
	}
#endif

	return i;
}

void 
Audioh::sid_delete(ALuint sid)
{
	if (verbose)
	printf("audioh: sid_delete(%d)\n",sid);

	if (checkSid(sid))
	this->sid[sid]=SID_NULL;
}

void
Audioh::sid_play(ALuint sid)
{
	if (verbose)
	printf("audioh: sid_play(%d)\n",sid);

#if OPENAL
	if (checkSid(sid))
	alSourcePlay(this->sid[sid]);
#endif
#if SDLSOUND
	if (checkSid(sid))
	Mix_Resume(sid);
#endif
}


void
Audioh::sid_stop(ALuint sid)
{
	if (verbose)
	printf("audioh: sid_stop(%d)\n",sid);

#if OPENAL
	if (checkSid(sid))
	alSourceStop(this->sid[sid]);
#endif
#if SDLSOUND
	if (checkSid(sid))
	{
	Mix_HaltChannel(sid);
	}
#endif
}

void 
Audioh::sid_pause(ALuint sid)
{
	if (verbose)
	printf("audioh: sid_pause(%d)\n",sid);

#if OPENAL
	if (checkSid(sid))
	alSourcePlay(this->sid[sid]);
#endif
#if SDLSOUND
	if (checkSid(sid))
	Mix_Pause(sid);
#endif
}

void 
Audioh::sid_loop(ALuint sid, bool loop)
{
	if (verbose)
	printf("audioh: sid_loop(%d,%d)\n",sid,loop);

#if OPENAL
	if (checkSid(sid))
	alSourcei (this->sid[sid], AL_LOOPING, (int)((loop)?1:0));
#endif
#if SDLSOUND
	// TODO - sid_loop for sdl
#endif
}

void
Audioh::sid_pos(ALuint sid,
		float x,float y, float z)
{
	this->sid_pos(sid,x,y,z,x,y,z);
}

void
Audioh::sid_pos(ALuint sid,
		float x, float y, float z,
		float x1, float y1, float z1)
{
	float ori[6]={ x,y,z,x1,y1,z1 };

#if OPENAL
	if (checkSid(sid))
	{
	alSource3f(this->sid[sid], AL_POSITION, x,y,z);
	alSourcefv(this->sid[sid], AL_ORIENTATION, ori );
	}
#endif
}

void
Audioh::sid_cone(ALuint sid, int angle, float in, float out)
{
#if OPENAL
	if (checkSid(sid))
	{
	alSourcei(this->sid[sid],AL_CONE_OUTER_ANGLE, angle);
	alSourcef(this->sid[sid],AL_CONE_OUTER_ANGLE, out);
	alSourcef(this->sid[sid],AL_CONE_INNER_ANGLE, in);
	}
#endif
}

// Get Info about SiDs //
// =================== //

int Audioh::sid_state(ALuint sid)
{
	int state;

#if OPENAL

	if (sid==SID_NULL||sid>this->n_sid) return 0;
	if (alIsSource(this->sid[sid])==AL_FALSE)
		return AL_FALSE;
	state=AL_INITIAL;
	alGetSourceiv(this->sid[sid],AL_SOURCE_STATE,&state);

	// #include <AL/altypes.h>
	// state could be: AL_PLAYING, AL_PAUSED, AL_STOPPED
	// or other not enought interesting.
	switch(state)
	{
	case AL_PLAYING:
		state=SOUND_PLAY; break;
	case AL_PAUSED:
		state=SOUND_PAUSE; break;
	case AL_STOPPED:
		state=SOUND_STOP; break;
	default:
		state=SOUND_NULL; break;
	}
	return state;
#endif

#if SDLSOUND
	if (Mix_Playing(sid))
		return SOUND_PLAY;
	if (Mix_Paused(sid))
		return SOUND_PAUSE;
/* XXX SDL cannot be sure of that  resid then?
	return SOUND_STOP;
*/
	return SOUND_NULL;
#endif

	return SOUND_NULL;
}

void Audioh::stopAll()
{
	unsigned int i=0;

	if (verbose)
	printf("audioh: stopAll()\n");

	for (i=0;i<n_sid;i++)
	if (sid_state(i)==SOUND_PLAY)
		sid_stop(i);
}

int Audioh::getNumBuffers()
{
	return n_buf;
}

int Audioh::getNumSources(){
	return n_sid;
}

char *Audioh::getFileName(ALuint sound)
{
	if (sound>=0 && sound <n_buf) return name[sound];
	else return "NO NAME";
}

