#ifndef _CONTROLSKEYBOARD_H_
#define _CONTROLSKEYBOARD_H_

/*! \file  ControlsKeyboard.h
 *  \brief ControlsKeyboard class declaration
 */

#if WIN32
#include <windows.h>
#endif

#include <SDL/SDL.h>

#include "Camera.h"
#include "Console.h"
#include "Message.h"
#include "Networkh.h"
#include "Collision.h"
#include "Stage.h"

//! Keyboard device controller
/*! This class controls the inputs from a keyboard. */
class ControlsKeyboard
{
public:
	//! ControlsKeyboard contructor
	ControlsKeyboard (Camera *cam, Stage *stg, Collision *col, Networkh *net);
	//! ControlsKeyboard destructor
	~ControlsKeyboard ();
	//! Grabs the input from the keyboard
	void input ();

	//! Save a config file
	int saveConfig (char *filename);

#warning Falten els get/set de les tecles per a configuracio dinamica

private:
	Camera   *cam;
	Console  *console;
	Collision *col;
	Message  *msg;
	Networkh *net;
	Stage    *stg;
	int jump;
	bool gojump;
	unsigned int jumpTime;

	//! Possible function keys
	enum FKey {
		WALK_F, WALK_B, RUN, TURN_L, TURN_R, STRAFE_L, STRAFE_R, JUMP,
		CENTER_V, LOOK_UP, LOOK_DOWN,
		FIRE, CHWEAP,
		MAP, TALK, SCREENSHOT, PAUSE, QUIT, FOVp, FOVm,
		SHOWPOS, SHOWITEMS, SHOWTHIRDP, KILL,
		MODE_PORTAL, MODE_BOUNDING, MODE_PAINTING, MODE_TEXTURES, 
		MODE_FOG, MODE_LIGHT, MODE_COL,
		FLY_UP, FLY_DOWN,
		NKEYS
	};

	//! List of possible key states
	enum KeyState {
		KEY_PRESSED=SDL_PRESSED, KEY_RELEASED=SDL_RELEASED
	};

	//! This struct defines a functional key and its default associated key and callback
	struct _key_t {
		//! glH function key name
		const char *name;
		//! SDL-equivalent associated key
		SDLKey sdl;
		/* TODO
		 * ugly hack to prevent keyboard repeating 
		 * circumvent the miswork of SDL_EnableKeyRepeat */
		//! Know if the key was already pressed (detect repeating)
		KeyState pressed;
		//! A hook to the glH key function callback
		void (ControlsKeyboard::*hook)(bool active, Movement *movement, Orientation *orientation);
	};

	//! Default key translation table
	static const _key_t default_keys[NKEYS];
	//! Actual key translation table
	_key_t *configured_keys;
#warning Should it be an object attribute? (probably NOT)
	//! SDL Keys
	Uint8 *keys;

	//! Look if a function key is pressed
	bool getIsFKeyPressed (FKey key);
	//! Look if a function key was already pressed on our key translation table
	bool getWasFKeyPressed (FKey key);
	//! Set the state of a function key (pressed/released) on our key translation table
	void setFKeyState (FKey key, KeyState state);

	// Define the callbacks
	void f_walk_f (bool active, Movement *movement, Orientation *orientation);
	void f_walk_b (bool active, Movement *movement, Orientation *orientation);
	void f_run (bool active, Movement *movement, Orientation *orientation);
	void f_turn_l (bool active, Movement *movement, Orientation *orientation);
	void f_turn_r (bool active, Movement *movement, Orientation *orientation);
	void f_strafe_l (bool active, Movement *movement, Orientation *orientation);
	void f_strafe_r (bool active, Movement *movement, Orientation *orientation);
	void f_jump (bool active, Movement *movement, Orientation *orientation);
	void f_center_v (bool active, Movement *movement, Orientation *orientation);
	void f_look_up (bool active, Movement *movement, Orientation *orientation);
	void f_look_down (bool active, Movement *movement, Orientation *orientation);
	void f_fire (bool active, Movement *movement, Orientation *orientation);
	void f_chweap (bool active, Movement *movement, Orientation *orientation);
	void f_map (bool active, Movement *movement, Orientation *orientation);
	void f_talk (bool active, Movement *movement, Orientation *orientation);
	void f_screenshot (bool active, Movement *movement, Orientation *orientation);
	void f_pause (bool active, Movement *movement, Orientation *orientation);
	void f_quit (bool active, Movement *movement, Orientation *orientation);
	void f_fovp (bool active, Movement *movement, Orientation *orientation);
	void f_fovm (bool active, Movement *movement, Orientation *orientation);
	void f_showpos (bool active, Movement *movement, Orientation *orientation);
	void f_showitems (bool active, Movement *movement, Orientation *orientation);
	void f_showthirdp (bool active, Movement *movement, Orientation *orientation);
	void f_kill (bool active, Movement *movement, Orientation *orientation);
	void f_mode_portal (bool active, Movement *movement, Orientation *orientation);
	void f_mode_bounding (bool active, Movement *movement, Orientation *orientation);
	void f_mode_painting (bool active, Movement *movement, Orientation *orientation);
	void f_mode_textures (bool active, Movement *movement, Orientation *orientation);
	void f_mode_fog (bool active, Movement *movement, Orientation *orientation);
	void f_mode_light (bool active, Movement *movement, Orientation *orientation);
	void f_mode_col (bool active, Movement *movement, Orientation *orientation);
	void f_fly_up (bool active, Movement *movement, Orientation *orientation);
	void f_fly_down (bool active, Movement *movement, Orientation *orientation);
	void f_delete (bool active, Movement *movement, Orientation *orientation);
	void f_accept (bool active, Movement *movement, Orientation *orientation);
	void f_cancel (bool active, Movement *movement, Orientation *orientation);

	void saveScreenShot();
	void walk (float dir, Movement *movement);
	void turn (float dir, Orientation *orientation);

	//! Load a config file
	int loadConfig (char *filename, _key_t *keys);

	// XXX - * -> variables/functions not related to controls
	// *
	int animation;
	bool running;
	float step,stepVar;
	bool moving;

	float mouse_speed_x, mouse_speed_y;
	float speed;
	float turn_speed;

	int numLegsState;
	bool *legsState;
	int weaponStyle;
	unsigned int nextShoot;
	int xc,yc;
	bool *keyPressed;
	int k;
	double pi;
	double yaw,yawV;

	void strafe(float dir, Movement *movement);

	// *
	void setAnimation(int which, int part);
};

#endif

