//#include "stdafx.h"
#include "GLImage.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GLImage::GLImage()
{
	m_width  = 0; 
	m_height = 0;
	m_format = GL_RGBA;
	m_type   = GL_UNSIGNED_BYTE;

	m_data   = 0;
}

GLImage::~GLImage()
{
	/*
	if (m_data) delete m_data;
	m_data = 0;
	*/
	clear();
}

//////////////////////////////////////////////////////////////////////
// Member functions
//////////////////////////////////////////////////////////////////////

void GLImage::set(GLsizei width, GLsizei height, GLenum format, GLenum type, GLubyte* data)
{
	m_width  = width;
	m_height = height;
	m_format = format;
	m_type   = type;
	m_data	 = data;
}

void GLImage::setFormat(GLenum format)
{
	m_format = format;
}
void GLImage::setType(GLenum type)
{
	m_type = type;
}

GLsizei GLImage::getWidth()
{
	return m_width;
}

GLsizei GLImage::getHeight()
{
	return m_height;
}
GLenum GLImage::getFormat()
{
	return m_format;
}
GLenum GLImage::getType()
{
	return m_type;
}
GLubyte* GLImage::getData()
{
	return m_data;
}

void GLImage::operator=(const GLImage &image)
{
	if(m_data != 0) delete[] m_data;

	m_width  = image.m_width;
	m_height = image.m_height;
	m_format = image.m_format;
	m_type   = image.m_type;


	long dataSize;

	switch(m_format)
	{
		case GL_RGB:	dataSize = m_width * m_height * 3;break;
		case GL_RGBA:	dataSize = m_width * m_height * 4;break;
	}

	m_data = new GLubyte[dataSize];

	for(int i = 0 ; i < dataSize ; i++)
	{
		m_data[i] = image.m_data[i];
	}
}
bool  GLImage::createFromBuffer(int x, int y, int width, int height)
{
	if (m_data != 0 ) delete[] m_data;
	
	m_data = 0;

	long dataSize;

	m_width  = width;
	m_height = height;

	switch(m_format)
	{
		case GL_RGB:	dataSize = width*height*3;break;
		case GL_RGBA:	dataSize = width*height*4;break;
	}
	m_data = new GLubyte[dataSize];

	if (m_data == 0) return false;
	
	glReadPixels( x, y, m_width, m_height, m_format, m_type, m_data);

	return true;
}

void GLImage::draw(GLfloat posX, GLfloat posY, GLfloat posZ)
{
	if(m_data != 0)
	{
		glRasterPos3f(posX, posY, posZ);
		glDrawPixels(m_width, m_height, m_format, m_type, m_data);
	}
}

void  GLImage::drawZoom(GLfloat zoomX, GLfloat zoomY)
{
	glPixelZoom(zoomX, zoomY);	
}


bool GLImage::resize(GLsizei width, GLsizei height)
{
	GLubyte *data_out = 0;

	long dataSize;

	switch(m_format)
	{
		case GL_RGB:	dataSize = width * height * 3;break;
		case GL_RGBA:	dataSize = width * height * 4;break;
	}

	data_out = new GLubyte[dataSize];

	if(data_out == 0) return false;

	gluScaleImage(m_format,m_width, m_height,m_type,m_data,width,height,m_type,data_out);
	
	delete m_data;

	m_width = width;
	m_height = height;
	m_data = data_out;

	return true;
}
void GLImage::clear()
{
	if(m_data != 0) delete[] m_data;
	m_data   = 0;
	m_width  = 0;
	m_height = 0;
	m_format = GL_RGBA;
	m_type   = GL_UNSIGNED_BYTE;
}

bool GLImage::setAlphaEqualRGB()
{

	if ( m_format != GL_RGBA ) return false;
	for ( int i = 0; i < m_width * m_height * 4; i+=4) {
		m_data[i+3] = (m_data[i] + m_data[i+1] + m_data[i+2])/3;//abans copiavem el vermell. S'ha de mirar com fer-ho
	}
	return true;
}



