#include "MainMenu.h"

MainMenu::MainMenu()
{
	register int i;
	char name[1024];

	keypressed=false;

	option=0;
	esc=false;

	/* init opengl */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(90.0,0.0,0.0,500.0);

	/* modelview */
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LESS);
	glEnable(GL_BLEND);

	/* camera */
	gluLookAt(
		0,0,-100,
		0,0,0,
		0,1,0);

	/* load textures */
	int w,h,bpp;
	unsigned char *ptr;

	for (i=0;i<MENU_LAST;i++)
	{
		try {
		sprintf(name,"menu%d.png",i);
		/* load image */
		menu[i]=TGAloader_tid(name);
		} catch(int e) {
			printf("menu%d.png not found\n",i);
		}
	}

	/* Load LOGO */
	try {
		logo=TGAloader_tid("logo.jpg");
	} catch(int e) {
		printf("perror not found %d\n",e);
	}
}

MainMenu::~MainMenu()
{
	/* free textures */
	glDeleteTextures(4,menu);
	menu[0]=logo;
	glDeleteTextures(1,menu);
}

void MainMenu::drawScene()
{
	unsigned char *c_menu0;
	int i,w,h,bpp;

	/* scene */
	glClearColor(0.0,0.0,0.0,1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	
	/* load textures */
	glLoadIdentity();

	glEnable(GL_TEXTURE_2D);
	/* glh logo */ /* 150x100 */ //aka 3x2
	glBindTexture(GL_TEXTURE_2D,logo);
	glColor3f(1.0,1.0,1.0);
	glBegin(GL_QUADS);
		glTexCoord2f(1,0);glVertex3f(-0.2,0.9,0.0);
		glTexCoord2f(0,0);glVertex3f(-0.9,0.9,0.0);
		glTexCoord2f(0,1);glVertex3f(-0.9,0.2,0.0);
		glTexCoord2f(1,1);glVertex3f(-0.2,0.2,0.0);
	glEnd();

	/* images must be 10x2 scaled (500x100) */

	/* menu options */
	for(i=0;i<MENU_LAST;i++) 
	{
	glBindTexture(GL_TEXTURE_2D,menu[i]);
	glColor3f(1.0,1.0,1.0);
	glBegin(GL_QUADS);
		glTexCoord2f(1,0);glVertex3f(0.4,0.2-(i*0.2),0.0);
		glTexCoord2f(0,0);glVertex3f(-0.6,0.2-(i*0.2),0.0);
		glTexCoord2f(0,1);glVertex3f(-0.6,-(i*0.2),0.0);
		glTexCoord2f(1,1);glVertex3f(0.4,-(i*0.2),0.0);
	glEnd();
	}

	/* selector */
	glBindTexture(GL_TEXTURE_2D,menu[0]);
	glBegin(GL_TRIANGLES);
		glColor3f(1.0,1.0,1.0);
		glTexCoord2f(.2,.2);
		glVertex3f(-0.6,0.1-option*0.2,0.0);
		glTexCoord2f(.2,0);
		glVertex3f(-0.8,0.2-option*0.2,0.0);
		glTexCoord2f(0,.2);
		glVertex3f(-0.8,0.0-option*0.2,0.0);
	glEnd();
}

void MainMenu::setOption(int n)
{
	option=n;
	if (option<0) option=MENU_LAST-1;
	if (option==MENU_LAST) option=0;
}

void MainMenu::getKeys()
{
	Uint8 *keys;
	int x,y;

	/* fetch user events */
	SDL_PumpEvents();
	//SDL_GetRelativeMouseState(&x,&y);
	keys=SDL_GetKeyState(NULL);
	if (keys[SDLK_1])
		{
		esc=true;
		setOption(MENU_OFFLINE);
		}
	if (keys[SDLK_ESCAPE])
		{
		esc=true;
		option=MENU_EXIT;
		//setOption(MENU_EXIT);
		}

	if (keys[SDLK_RETURN])
		{
		esc=true;
		}

	if (keypressed==false)
	{
	if (keys[SDLK_UP]==SDL_PRESSED)
		{
		setOption(option-1);
		keypressed=true;
		}
	if (keys[SDLK_DOWN]==SDL_PRESSED)
		{
		setOption(option+1);
		keypressed=true;
		}
	}
	if (! keys[SDLK_UP] && !keys[SDLK_DOWN] )
	{
		keypressed=false;
	}
}

bool MainMenu::isFinished()
{
	return esc;
}

int MainMenu::getOption()
{
	printf("OPTION: %d\n",option);
	return option;
}

