#include "Networkh.h"

#if WIN32
#include "windows.h"
#include "winsock.h"
#endif

int NETWORK=0; // network enabled by default

Stage* Networkh::getStage()
{
	return stg;
}

int Networkh::do_send(int s, char *ptr, int len)
{
	int sent=0,ret;

	if (!CONECTED) return 0;

	do {
		ret=send(s,ptr+sent,len-sent,0);
		if (ret==-1) {
#if !WIN32
			shutdown(s,SHUT_RDWR);
#endif
			close(s);
			CONECTED=false;
			return -1;
		}
		sent+=ret;
	} while(sent<len);

	return len;
}

int Networkh::do_recv(int s, char *ptr, int len)
{
	int rect=0,ret;

	if (!CONECTED) return 0;

	do {
		ret=recv(s,ptr+rect,len-rect,0);
		if (ret==-1) {
#if !WIN32
			shutdown(s,SHUT_RDWR);
#endif
			close(s);
			CONECTED=false;
			return -1;
		}
		rect+=ret;
	} while(rect<len);

	return len;
}

Networkh::Networkh(char *hostname, char *login, char *pass, int port,int bsize)
{
	RECIVING=true;
	CONECTED=false;
	struct	sockaddr_in servaddr;
	struct  hostent *he;
#if WIN32
	WSADATA              wsa;
	WORD                 wVersionRequested;
	wVersionRequested = MAKEWORD( 2, 0 );
	if (WSAStartup(wVersionRequested , &wsa)!=0) 
	{
		printf("[n] Error starting WSA.\n");
		throw 1;
	}
#endif

	strcpy (this->nickSend,login);
	strcpy (this->pass,pass);

	this->hostname=hostname;
	this->port=port;
	this->stg=stg;
	this->bsize=bsize;

	printf("[n] Loading networkh-client version %s...\n",VERSION);

	he=(struct hostent *)gethostbyname(hostname);
	if (he==(struct hostent *)0)
		{
		fprintf(stderr,"Cannot resolve hostname\n");
		throw 1;
		}

	if ( (s = socket(AF_INET, SOCK_STREAM, 0)) < 0 )
		{
		fprintf(stderr, "Cannot create socket.\n");
		throw 1;
		}
	
	servaddr.sin_family      = AF_INET;
	memcpy((char *)&servaddr.sin_addr, he->h_addr, he->h_length);
	servaddr.sin_port        = htons(port);

	if (  connect (s, (struct sockaddr *) &servaddr, sizeof(servaddr)) )
	{
		printf("[n] Cannot connect to remote host\n");
		throw 1;
	}

	printf("[n] Connected to host %s at port %d.\n",hostname,port);
	
	printf("[n] Launching networkh thread.\n");
	SDL_CreateThread(useThread,this);
}

void Networkh::setStage(Stage *stg)
{
	this->stg=stg;
	this->cli_sendOkOk();
}


int Networkh::start(void *mmh)
{
	printf("[n] Networkh thread launched\n");

	while ( 1 )
	{
		int n;
		char buf[1];
		n=do_recv(s,buf,1);
		   if (n<0)	{
			perror("recv");
			fprintf(stderr,"Error on recv (%d).\n",n);
			killProgram(0);
			}
		   if (!n) {
			   perror("recv=0; waiting..\n");
			   continue;
		   }
		switch(buf[0])
		{
		case PACKET_VERS: cli_reciveVers();	break;
		case PACKET_PKEY: cli_reciveKey();	break;
		case PACKET_ERRR: cli_packetErr();	break;
		case PACKET_TALK: cli_reciveTalk();	break;
		case PACKET_CHKF: cli_checkFile();	break;
		case PACKET_FILE: cli_saveFile();	break;
		case PACKET_MOVE: cli_reciveMove();	break;
		case PACKET_PART: cli_recivePart();	break;
		case PACKET_ITEM: cli_reciveItem();	break;
		case PACKET_BEGN: cli_reciveBegin();	break;
		case PACKET_WEAP: cli_reciveWeapon();	break;
		case PACKET_JOIN: cli_reciveJoin();	break;
		case PACKET_OKOK: cli_reciveOkOk();	break;
		case PACKET_DEPS: cli_killFlag(); break;
		case PACKET_SHOT: cli_reciveShot(); break;
		case PACKET_ROLE: cli_reciveRole(); break;
		default:
			fprintf(stderr,"Unknown data recived %d :/\n",buf[0]);
			killProgram(0);
			break;
		}
	}
	return 1;
}


Networkh::~Networkh()
{
#if WIN32
	closesocket(s);
#else
	close(s);
#endif
	printf("[n] Connection closed.\n");
}



int useThread(void *net){
	Networkh *mmh=(Networkh *)net;
	mmh->start(NULL);
	return 1;
}

//===================
//--               --
//-- Packet Recive --
//--               --
//===================

struct Role { unsigned char life,ammo, sh_head,sh_body,strength,def,aim,speed; };
void Networkh::cli_reciveRole()
{
	Role role;
	char *b=(char *)&role;
	do_recv(s, b, sizeof(Role));

	stg->getCamaraInterface()->setHealth((int)role.life);
	stg->getCamaraInterface()->setMunition((int)role.ammo);
	stg->getCamaraInterface()->setHelmet((int)role.sh_head);
	stg->getCamaraInterface()->setStrength((int)role.strength);
	stg->getCamaraInterface()->setDefense((int)role.def);
	stg->getCamaraInterface()->setAim((int)role.aim);
	stg->getCamaraInterface()->setVelocity((int)role.speed);
	
}

void Networkh::cli_packetErr()
{
	char b[1];
	char *str;
	
	do_recv(s,b,1);
	str=(char *)malloc(b[0]+10);
	do_recv(s,str,b[0]);
	str[b[0]]=0;
	printf("Packet error, server says: \"%s\".\n",str);
	free(str);

	killProgram(0);
}

void Networkh::cli_reciveItem()
{
	char b[2];
	do_recv(s,b,2);
	printf("ITEM %d GIVE %d\n",b[0],b[1]);
}

void Networkh::cli_checkFile()
{
	char *b,
	     *str,
	     *crc;
	unsigned int i;
	FILE *fd;
		
	b=(char *)malloc(5);
	str=(char *)malloc(300);

	RECIVING=true;
	printf("reciving files\n");
	do_recv(s,b,1);
	do_recv(s,str,b[0]);
	str[b[0]]=0;
	printf("   - Checking file %s...",str);
	try {
	chdir (makeDirs(str));
	} catch (int e) {
		printf("cannot create directory\n");
		return;
	}
	fd=fopen(str,"rb"); 
	if (!fd)
	{
		printf("not found\n");
		b[0]=PACKET_CRCF; // reply my crc
		memset(b+1,0,4); // null crc body
		do_send(s,b,5); // send my crc.
	} else {
		fseek(fd,0L,SEEK_END);
		i=ftell(fd);
		if (!i) i=20;
		fseek(fd,0L,SEEK_SET);
		str=(char*)realloc(str,i); // resize baffah!
		if (!str)
		{
			printf("Error reallocating %d :/\n",i);
			exit(1);
		}
		fread(str,i,1,fd);
		i = htonl(crc32(str,i)); // i=crc result.
		crc=(char *)&i;
		printf("found as crc [%2x:%2x:%2x:%2x]\n",
			(unsigned char)crc[0],
			(unsigned char)crc[1],
			(unsigned char)crc[2],
			(unsigned char)crc[3]);
		// Change for better code.
		str[0]=PACKET_CRCF;
		/*
		memcpy(str+1,crc,4);
		send(s,str,5,0);
		*/
		do_send(s,str,1); // send packet type.
		do_send(s,crc,4); // Send crc
	}
	free (str);
	free(b);
}


void Networkh::cli_saveFile()
{
	char *b,fileName[50];
	unsigned int *i;
	unsigned long I,n,j;
	FILE *fd;

	b=(char *)malloc(bsize);
	if (!b)
	{
		printf("Error on malloc\n");
		exit(1);
	}
	do_recv(s,b,1);
	j=(unsigned int)b[0];
	  do_recv(s,b,j);
	  b[j]=0;
	  printf("   - downloading file %s",b);
	  chdir(makeDirs(b));
	  strcpy(fileName,b);
	
	  do_recv(s,b,4); // get file size
	  i=(unsigned int *)b;
	  j=ntohl(*i);
	  printf("    Size of it is: %d\n",j);
	  printf("Bsize: %d\n",bsize);
	
	  fd=fopen(fileName,"w+b"); // ARRGGHHH! odio a WINDOWS!!!
	if (!fd) 
	{
		printf("[e] Cannot open file for writing ?!? \n");
		exit(1);
	}

	  for(	I=n=0; 
		I<j;  
		I+=(I+bsize>j)?j-I:bsize,n++
	     )
	  {
		do_recv(s,b,(I+bsize>j)?j-I:bsize);
		fwrite(b,(I+bsize>j)?j-I:bsize,1,fd);
		//write(1,b,(80));
	//	sleep(2);
	  }
	  
	  printf("    Recived %d bytes\n",j);
	fclose(fd);
	free(b);
}

void Networkh::cli_reciveMove()
{
	char pkt[16];
	char who;
	unsigned char rot,ani;
	float *x,*y,*z,rot2;
	//printf("RECIVE MOVE\n");

	memset(pkt,0,15); // fucking null.
	do_recv(s,pkt,15);

	x=(float *)(pkt+1);
	y=(float *)(pkt+5);
	z=(float *)(pkt+9);
	
	who=pkt[0];
	rot=pkt[13];
	ani=pkt[14];

	//printf("Recieved: %d\n",rot);
	rot2=(float)(rot*2);

	//printf("Recive: Who: %d Pos: %f %f %f Rot: %f\n",who,*x,*y,*z,rot2);
	if (stg->getCamaraDNI()==who){
		stg->setCamaraPos(GL3f(*x,*y,*z));
	}
	else {
		stg->getEnemy(who)->setPos(GL3f(*x,*y,*z));
		stg->getEnemy(who)->setRotation(rot2);
		stg->getEnemy(who)->setAnimation((ani&3),((ani&28)>>2),((ani&224)>>5));
	} 
	/*
	printf("who %d\n",who);
	printf("rotation %d\n",rot);
	printf("animation is %d\n",ani);
	printf("x,y,z are %f %f %f\n",*x,*y,*z);
	*/
}

void Networkh::cli_reciveVers()
{
	char b[2];
	char *str;
	do_recv(s,b,2);
	str=(char *)malloc(b[1]+1);
	do_recv(s,str,b[1]);
	str[b[1]]=0;
	printf("Version: %d (banner: %s)\n",b[0],str);
	free(str);
	cli_sendUser(nickSend);
}

void Networkh::cli_reciveKey()
{
	printf("Reciving PACKET KEY\n");
	do_recv(s,key,4);
	printf("Key recived: [%x:%x:%x:%x]\n",
			(unsigned char)key[0],(unsigned char)key[1],(unsigned char)key[2],(unsigned char)key[3]);
	cli_sendPass(pass);
}

void Networkh::cli_reciveWeapon()
{
	char b[3];
	do_recv(s,b,3);
	printf("Change User %d weapon to %d, munition %d\n",b[0],b[1],b[2]);
}

void Networkh::cli_reciveTalk()
{

	char b[2],*str;
	do_recv(s,b,2);
	str=(char *)malloc(b[1]+1);
	do_recv(s,str,b[1]);
	str[b[1]]=0;
	stg->getMessageControl()->addMessage(str);
	printf("TALK: %d %s (%d)\n",b[0],str,b[1]);
	free(str);
}

void Networkh::cli_recivePart()
{
	char b[1];
	char who[100];
	do_recv(s,b,1);
	sprintf(who,"%s parts the game.\n",stg->getEnemy(b[0])->getNick());
	stg->killEnemy(b[0]);	
	stg->getMessageControl()->addMessage(who);

}

void Networkh::cli_reciveOkOk()
{
	if (CONECTED) cli_sendOkOk();
	CONECTED=true;
	printf("Deps loaded\n\n");

}

void Networkh::cli_reciveBegin()
{
	GL3f pos;
	float x,y,z,r;
	char hab;
	char b[17];
	do_recv(s,b,17);

	printf("Begin recived, game must begin!\n");

	memcpy(&x,b,4);
	memcpy(&y,b+4,4);
	memcpy(&z,b+8,4);
	memcpy(&r,b+12,4);
	memcpy(&hab,b+16,1);

	pos=GL3f(x,y,z);
	stg->setCamaraPos(pos,false);

	//if (NETWORK)
	//_posCam=pos;

	pos.print();
	//stg->setCamaraObj(obj.x,obj.z);
//	stg->setCamaraId(hab);
	printf("YOU ARE HERE: %f %f %f r:%f h:%d\n",x,y,z,r,hab);
}


void Networkh::cli_reciveJoin()
{
	char pkt[3],*b,*c;
	char who;
	printf("RECIVE JOIN\n");

//	memset(pkt,0,15); // fucking null.
	do_recv(s,pkt,2);
	
	who=pkt[0];
	b=(char *)malloc(pkt[1]+1);
	
//	memset(b,0,pkt[1]+1);
	do_recv(s,b,pkt[1]); // Get Nick
	b[pkt[1]]=0;
	printf("Nick is: %d %s\n",who,b);

	do_recv(s,pkt,1);
	c=(char *)malloc(pkt[0]+1);
//	memset(b,0,pkt[0]+1);
	do_recv(s,c,pkt[0]);
	c[pkt[0]]=0;
	printf("RPH is: %s\n",c);
	
	if (!strcmp(b,nickSend)){
		if (strcmp(stg->getCamaraNick(),b)){
			stg->setCamaraNick(b);
			stg->setCamaraDNI(who);
			printf("You've been logged in the glh server :)\n");
		}
		else printf("You alredy exists xD	..\n");
	}
	else{
		if(!stg->existsEnemy(who)){
			printf("Enemy %d added (%s)\n",who,b);
			stg->addEnemy("goblin.car",b,who);
		}
		else printf("Enemy alredy exists\n");
	}

	free(b);
	free(c);
	return;
}

void Networkh::cli_reciveShot(){
	char b[2];
	do_recv(s,b,2);
	printf("Shot throwed by %d with weapon id %d\n",b[0],b[1]);
}

int Networkh::cli_sendUser(char *user){

	if (strlen(user)<=0) return 0;

	char b[300];
	
	b[0]=PACKET_USER;
	b[1]=strlen(user);
	memcpy(b+2,user,b[1]);
	do_send(s,b,b[1]+2);
	printf("Nick %s length: %d\n",user,b[1]);
	
	return 1;
}
		
int Networkh::cli_sendPass(char *pass1){

	
	unsigned int crc;
	char *b;
	char passCrc[100],pass[100];
	sprintf(pass,"%s\n",pass1);
	
	int passLength=strlen(pass);
	if (passLength<=0) return 0;
	strcpy(passCrc+1,pass);
	passCrc[0]=PACKET_PASS;
	for(int i=0;i<4;i++)
		for(int j=0; j<passLength;j++)
			passCrc[j+1]^=key[i];

	crc=crc32(passCrc+1,passLength-1); 
	crc=htonl(crc); // inter-architecture save


	b=(char *)&crc;
	memcpy(passCrc+1,b,4);

	do_send(s,passCrc,5);
	printf("KEY SENT: [%2x:%2x:%2x:%2x]\n",
			(unsigned char)b[0],
			(unsigned char)b[1],
			(unsigned char)b[2],
			(unsigned char)b[3]);
		
	return 1;
}
		
int Networkh::cli_sendTalk(const char *talk)
{
	if (strlen(talk)<=0) return 0;

	char b[300];
	b[0]=PACKET_TALK;
	b[1]=strlen(talk);
	strcpy(b+2,talk);
	do_send(s,b,b[1]+2);

	return 1;
}

int Networkh::cli_sendNick(char *nick){

	if (strlen(nick)<=0) return 0;

	char b[300];

	b[0]=PACKET_NICK;
	b[1]=strlen(nick);
	memcpy(b+2,nick,b[1]);
	do_send(s,b,b[1]+2);

	return 1;
}

int Networkh::cli_sendMove(GL3f pos,int rot, int ani){
	int n=0;
	if (RECIVING) return 1;	
	char b[300];
	b[0]=PACKET_MOVE;
//		printf("Send: %f - %f - %f (Rot:%d, Ani:%d) \n",pos.x,pos.y,pos.z,rot,ani);
	// ?to?? sux on float.
	//x=htonl(x); y=htonl(y); z=htonl(z); // convert to network data
	// Generate packet
	memcpy(b+1,&pos.x,4); memcpy(b+5,&pos.y,4); memcpy(b+9,&pos.z,4);
	b[13]=(unsigned char)(rot/2);
//	printf("Send: %d\n",(unsigned char)rot);
	//printf("%d\n",b[13]);
	b[14]=(unsigned char)ani;
	do_send(s,b,15); // Send move packet

	return 1;
}

int Networkh::cli_sendWeap(int weap){

	char b[2];
	if (weap>=0)
	{
		b[0]=PACKET_WEAP; // change to weapon N.
		b[1]=weap;
		do_send(s,b,2);
	} else {
		b[0]=PACKET_CHWP; // change to the next weapon
		do_send(s,b,1);
	}


	return 1;
}

int Networkh::cli_sendOkOk(){
	char b[1]={PACKET_OKOK};
	printf("Ready for playing, waiting for server to start...\n");
	do_send(s,b,1);
	return 1;
}

bool Networkh::conected(){
	return CONECTED;
}

void Networkh::cli_killFlag(){
	printf("Deps ok\n");
	RECIVING=false;
}


struct Shot { float x,y,z,vx,vy,vz; };

void Networkh::cli_sendShot(float x,float y,float z,float vx,float vy, float vz)
{
	Shot shot;
	GL3f vDir=Normalize(GL3f(vx,vy,vz));
	shot.x=x;   shot.y=y;   shot.z=z;
	shot.vx=vDir.x; shot.vy=vDir.y; shot.vz=vDir.z;
	char *b=(char *)&shot;
	char buf[1+sizeof(Shot)];
	buf[0]=PACKET_SHOT;
	memcpy(buf+1,b,sizeof(Shot));	
	do_send(s,buf,1+sizeof(Shot));
}



