#include "Object3D.h"

Object3D::Object3D(char *file)
{
	animacioTotal=0;
	numFigures=-1;
	text=0;

	//Cargamos el modelo
	if ( loadMDH(file) == false )
	{
		printf("\n##### MODEL %s NOT FOUND #####\n\n",file);
		killProgram(0);
	}
}

Object3D::~Object3D()
{
	GLuint buf[2];
	buf[0]=text;
	glDeleteTextures(1,buf);

	delete[] textureVertex;
	
	for (int i=0;i<numFigures;i++)
		delete figura[i];

	delete[] figura;
	delete[] animacio;
}

bool Object3D::loadMDH(char *file)
{
	FILE* fd = NULL;
	char buf[100];
	int k;
	int animPos = 0;
	int conta = -1;
	float *vertexTempo1=0,*vertexTempo2=0;
	float *vertexTextureTempo=0;
	int *faceTempo=0;
	int *faceMapTempo=0;
 	int numVertex;
	GL3f vertex;
	GL3i facestemp;
	GL2f textureP;
	
	if (! chdir(getDirs(file)) )
	{
		fd=fopen (file,"r");
	} 

	if (fd == NULL) 
	{
		printf("[e] Object3D::Cannot open file %s\n",file);
		return 0;
	}
		
	/* Read MDH headers */
	do {
		buf[0]=0;
		fscanf(fd,"%s",buf);
		if (feof(fd))
		{
			printf("[e] Unexpected end of file loading '%s'\n",file);
			return 0;
		}

		if (!strcmp(buf,"&NUM_FRAMES"))
		{
			fscanf(fd,"%d",&numFigures);
			figura=new Figure*[numFigures];
		}
		else if (!strcmp(buf,"&NUM_ANIMATIONS"))
		{
			fscanf(fd,"%d",&animacioTotal);
			animacio=new int[(animacioTotal*2)+1];
		}
		else if (!strcmp(buf,"&TEXTURE"))
		{
			fscanf(fd,"%s",buf);
			setTexture(buf);
		}
		else if (!strcmp(buf,"&SPEED"))
		{
			fscanf(fd,"%d",&speedDefault);
		}
		else if (!strcmp(buf,"&MAP_PARTS"))
		{
			fscanf(fd,"%d",&numTextureVertex);
			//Este array servira posteriormente para colocar los vertices de textura del
			//vertexArray. Multiplicamos el numero de caras por 6 ya que cada cara tiene
			//3 vertices y cada vertice 2 componentes (u,v)
			textureVertex=new float[numFaces*6];
			vertexTextureTempo=new float[numTextureVertex*2];
			for (k=0;k<numTextureVertex*2;k+=2)
			{
				fscanf(fd,"%f %f ",
					&vertexTextureTempo[k],
					&vertexTextureTempo[k+1]);
			}
		}
		else if (!strcmp(buf,"&MAP_FACES"))
		{
			fscanf(fd,"%d",&numFaces);
			faceMapTempo=new int[numFaces*3];
			for (k=0;k<numFaces*3;k+=3)
			{
				fscanf(fd,"%d %d %d\n",
					&faceMapTempo[k],
					&faceMapTempo[k+1],
					&faceMapTempo[k+2]);
			}
		}
		else if (!strcmp(buf,"&FACES"))
		{
			fscanf(fd,"%d",&numFaces);
			faceTempo=new int[numFaces*3];
			for (k=0;k<numFaces*3;k+=3)
			{
				fscanf(fd,"%d %d %d\n",
					&faceTempo[k],
					&faceTempo[k+1],
					&faceTempo[k+2]);
			}
		}
		else if (!strcmp(buf,"&VERTEX")){
			fscanf(fd,"%d",&numVertex);
		}
	} while(buf[0]!='*');

	vertexTempo1=new float[numVertex*3];

	//Este array servira posteriormente para colocar los vertices del
	//vertexArray. Multiplicamos el numero de caras por 9 ya que cada cara tiene
	//3 vertices y cada vertice 3 componentes (x,y,z)
	vertexTempo2=new float[numFaces*9];

	//Leemos las animaciones
	for (register int cont=0;cont<numFigures && !feof(fd);cont++)
	{
		//Comprobamos si es una nueva animacion o una continuacion de otra
		if (buf[0]=='*')
		{
			//Guardamos que frame es el primero de esta animacion
			animacio[animPos]=cont;
			//Miramos de que tipo es y tambien lo guardamos
			if (buf[1]=='S')
			{
				animacio[animPos+1]=1;
			}
			else if(buf[1]=='D')
			{
				animacio[animPos+1]=0;
			}
			animPos+=2;
		}
		else if (buf[0]!='.')
		{
			return false;
		}

		//Creamos un nuevo objeto figure y le pasamos el numero de caras del modelo
		figura[cont] = new Figure(numFaces);

		//Leemos los vertices
		for (k=0;k<numVertex*3;k+=3)
		{
			fscanf(fd,"%f %f %f",
				&vertexTempo1[k],
				&vertexTempo1[k+1],
				&vertexTempo1[k+2]);
			vertexTempo1[k]*=RATIO;
			vertexTempo1[k+1]*=RATIO;
			vertexTempo1[k+2]*=RATIO;
		}

		//Ponemos los vertices ordenados por caras en el array correspondiente al vertexArray
		//Miramos uno por uno cada vertice de cada cara y ponemos sus 3 componentes en el
		//vertexArray
		for (k=0;k<numFaces*3;k++)
		{
			vertexTempo2[(k*3)]=vertexTempo1[faceTempo[k]*3];
			vertexTempo2[(k*3)+1]=vertexTempo1[(faceTempo[k]*3)+1];
			vertexTempo2[(k*3)+2]=vertexTempo1[(faceTempo[k]*3)+2];
		}

		//Le pasamos los vertices al objeto figura correspondiente
		figura[cont]->addVertex(vertexTempo2);

		//Volvemos a leer el fichero en busca de un * o un .
		fscanf(fd,"%s",buf);
	}

	fclose(fd);

	//Creamos su bounding con los vertices del primer frame
	bounding=GLBounding(figura[0]->getVertex(),numFaces*3);

	//Ponemos los vertices de textura ordenados por caras en el array correspondiente 
	//al vertexArray de texturas.
	//Miramos uno por uno cada vertice de textura de cada cara y ponemos sus 2 componentes en el
	//vertexArray de texturas
	for (k=0;k<numFaces*3;k++)
	{
		textureVertex[(k*2)]=vertexTextureTempo[faceMapTempo[k]*2];
		textureVertex[(k*2)+1]=vertexTextureTempo[(faceMapTempo[k]*2)+1];
	}
	//Creamos una animacion ficticia que empiece en el frame siguiente al ultimo.
	//Servira para saber cuando acaba la ultima animacion
	animacio[animPos]=numFigures;

	//Liberamos memoria
	delete [] vertexTextureTempo;
	delete [] vertexTempo1;
	delete [] vertexTempo2;
	
	// All OK
	return true;
}

int Object3D::setTexture(char *name)
{
	if (text)
		return 0; // cannot overwrite a texture

	text = TGAloader_tid(name);

	return text;
}

int Object3D::getNumFigures()
{
	return numFigures;
}

int Object3D::getAnimacioTotal()
{
	return animacioTotal;
}

Figure **Object3D::getFigura()
{
	return figura;
}

float *Object3D::getTextureVertex()
{
	return textureVertex;
}

float *Object3D::getVertex(int ani)
{
	if (ani<0) ani=0;

	return figura[ani]->getVertex();
}

int Object3D::getSpeedDefault()
{
	return speedDefault;
}

GLuint Object3D::getTexture()
{
	return text;
}

GLBounding Object3D::getBounding()
{
	return bounding;
}

void Object3D::setBounding(GLBounding b)
{
	bounding = b;
}

int Object3D::getTipoAnimacio(int num)
{
	return animacio[(num*2)+1];
}

int Object3D::getFirstFrame(int num)
{
	return animacio[num*2];
}

int Object3D::getNumTextureVertex()
{
	return numTextureVertex;
}

int Object3D::getNumFaces()
{
	return numFaces;
}
