#include "Player.h"

Player::Player()
{
	bounding= GLBounding();

	//Creamos un array de 3. Un objeto para la cabeza, uno para el cuerpo y otro para las piernas
	model=new Properties*[3];
	arma=NULL;

	// set default values
	strcpy(nick,"");
};

Player::~Player()
{
	for (int i=0;i<3;i++)
		delete model[i];

	delete[] model;
	delete[] bodyUnion;
	delete[] legsUnion;

	printf("[<] Player %s\n",file);
}

// LOAD .CAR FILES
void Player::load(char *file)
{
	strncpy(this->file,file,100);

	if ( isBinaryCarFile(file) )
		loadBinary(file);
	else	loadPlain(file);
}

bool Player::isBinaryCarFile(char *file)
{
	FILE *fd;
	char buf[4];
	bool ret=false;

	fd=fopen(file,"r");
	if (fd)
	{
	fread(buf,4,1,fd);
	if (!memcmp(buf,"CAR\0",4))
		ret=true;
	fclose(fd);
	}

	return ret;
}

void Player::loadBinary(char *file)
{
	// TODO
	printf("binary load not yet implemented\n");
}

void Player::loadPlain(char *file)
{
	char tempo[100];
	int i,frames;
	int err;
	FILE *origen;

	err=chdir(getDirs(file));

	if (err)
	{
		printf("Cannot chdir\n");
		return ;
	}
		
	//Abrimos el archivo
	origen=fopen (file,"r");
	chdir("../..");

	if (!origen) {
		printf("MODEL %s NOT FOUND\n",file);
		killProgram(0);
	}

	//Buscamos la informacion de las 3 partes del personaje
	for (i=0;i<3 && !feof(origen);i++){
		fscanf(origen,"%s\n",tempo);
		if (!strcmp(tempo,"HEAD")){
			fscanf(origen,"%s\n",tempo);

			//Creamos el objeto correspondiente a la cabeza
			model[HEAD]=new Properties(tempo); 
		}
		else if (!strcmp(tempo,"BODY")){
			fscanf(origen,"%s\n",tempo);
		
			//Creamos el objeto correspondiente al cuerpo
			model[BODY]=new Properties(tempo);
		}
		else if (!strcmp(tempo,"LEGS")){
			fscanf(origen,"%s\n",tempo);
		
			//Creamos el objeto correspondiente a las piernas
			model[LEGS]=new Properties(tempo);
		}
	}

	do{
		fscanf(origen,"%s",tempo);
	}while (strcmp(tempo,"LEGS_UNION_POINTS") && !feof (origen));

	fscanf(origen,"%d",&frames);
	if (feof (origen)) {
		printf ("ERROR LOADING CHARACTER\n");
		killProgram(0);
	}

	//Llenamos el array con los diferentes puntos de union entre las partes
	//Hay uno para cada frame
	legsUnion = new GL3f[frames];
	for (i=0;i<frames;i++){
		fscanf(origen,"%f %f %f\n",&legsUnion[i].x,&legsUnion[i].y,&legsUnion[i].z);
		legsUnion[i]=legsUnion[i]*RATIO;
	}

	do{
		fscanf(origen,"%s",tempo);
	}while (strcmp(tempo,"BODY_UNION_POINTS") && !feof (origen));

	fscanf(origen,"%d",&frames);
	if (feof (origen)) {
		printf ("ERROR LOADING CHARACTER\n");
		killProgram(0);
	}

	bodyUnion = new GL3f[frames];
	weaponUnion= new GL3f[frames];
	weaponAngle= new GL2f[frames];
	for (i=0;i<frames;i++){
		fscanf(origen,"%f %f %f\n",
			&bodyUnion[i].x,
			&bodyUnion[i].y,
			&bodyUnion[i].z);
		bodyUnion[i]=bodyUnion[i]*RATIO;
	}	

	// Creamos el bounding general del personaje apartir de 
	// los boundings de las diferentes partes
	GLBounding boundingTemporal;

	boundingTemporal=model[0]->getObject3D()->getBounding();
	bounding.addBounding(boundingTemporal);

	boundingTemporal=model[1]->getObject3D()->getBounding();
	boundingTemporal=boundingTemporal + getUnion(1,0);
	bounding.addBounding(boundingTemporal);

	boundingTemporal=model[2]->getObject3D()->getBounding();
	boundingTemporal=boundingTemporal + getUnion(1,0) + getUnion(2,0);
	bounding.addBounding(boundingTemporal);

	do {
		fscanf(origen,"%s",tempo);
	}while (strcmp(tempo,"WEAPON_POS") && !feof (origen));

	for (i=0;i<frames;i++){
		fscanf(origen,"%f %f %f\n",
			&weaponUnion[i].x,
			&weaponUnion[i].y,
			&weaponUnion[i].z);
		weaponUnion[i]=weaponUnion[i]*RATIO;
	}

	do {
		fscanf(origen,"%s",tempo);
	} while (strcmp(tempo,"WEAPON_ANGLE") && !feof (origen));

	for (i=0;i<frames;i++)
		fscanf(origen,"%f %f\n",&weaponAngle[i].x,&weaponAngle[i].y);

	// put Player in the correct form
	update();

	printf("[>] Player %s\n",file);
}

GLBounding Player::getBounding()
{
	return bounding;
}

void Player::setPos(GL3f pos)
{
	model[LEGS]->setPos(pos);
}

GL3f Player::getPos()
{
	return model[LEGS]->getPos();
}

void Player::setRotation(float angle)
{
	model[LEGS]->setRotation(angle);
}

float Player::getRotation()
{
	return model[LEGS]->getRotation();
}

int Player::getId()
{
	return model[LEGS]->getId();
}

#if _PVS
void Player::setId(int id_old)
{
	model[LEGS]->setId(id_old);
}
#endif

bool Player::setAnimation(int animationHead, int animationBody, int animationLegs)
{
	//Comprobamos que la animacion sea valida
	if (animationLegs<0 || animationLegs>=model[LEGS]->getObject3D()->getAnimacioTotal()) return false;
	if (animationBody<0 || animationBody>=model[BODY]->getObject3D()->getAnimacioTotal()) return false;
	if (animationHead<0 || animationHead>=model[HEAD]->getObject3D()->getAnimacioTotal()) return false;

	//Activamos las animaciones
	model[LEGS]->setAnimation(animationLegs,1);
	if (animationBody==NOT_SHOOTING_FACE2FACE || animationBody==NOT_SHOOTING_2HAND || animationBody==NOT_SHOOTING_2HAND) model[BODY]->setAnimation(animationBody);
	else model[BODY]->setAnimation(animationBody,1);
	model[HEAD]->setAnimation(animationHead,1);

	animation  = 0;
	animation += animationHead;
	animation += animationBody<<2;
	animation += animationLegs<<5;

	return true;
}

int Player::getAnimation()
{
	return animation;
}

Properties *Player::getControl(int which)
{
	return model[which];
}

void addMovement (Movement &mov)
{
// XXX #warning This needs coordination with ControlKeyboard::<movement>
}

GL3f Player::getUnion (int part, int which)
{
	switch (part)
	{
		case LEGS:
			return model[LEGS]->getPos();
		case BODY:
			if (which<0) return legsUnion[model[LEGS]->getPosAnimacio()];
			else return legsUnion[which];
		case HEAD:
			if (which<0) return bodyUnion[model[BODY]->getPosAnimacio()];
			else return bodyUnion[which];
		case WEAPON_POS:
			if (which<0) return weaponUnion[model[BODY]->getPosAnimacio()];
			else return weaponUnion[which];
	}
	return GL3f (0,0,0);
}

GL2f Player::getWeaponRot (int which)
{
	if (which>=0) return weaponAngle[which];
	else return weaponAngle[model[BODY]->getPosAnimacio()];
}

void Player::update()
{
	model[LEGS]->update();
	model[BODY]->update();
	model[HEAD]->update();
	if (arma)
	arma->update();
}

void Player::setVisible(bool visible)
{
	model[LEGS]->setVisible(visible);
	model[BODY]->setVisible(visible);
	model[HEAD]->setVisible(visible);

}

bool Player::isVisible()
{
	return model[LEGS]->isVisible();
}

Weapon *Player::getWeapon()
{
	return arma;
}

void Player::setWeapon(Weapon *w,int numBalas)
{
	if (arma) delete arma;
	arma=w;
	actualAmmo=numBalas;
}

bool Player::haveWeapon()
{
	return (arma!=NULL);
}

void Player::killEnemy()
{
    model[LEGS]->setDesintegration(-1);
	model[BODY]->setDesintegration(-1);
	model[HEAD]->setDesintegration(-1);

    model[LEGS]->setExplosion(true);
	model[BODY]->setExplosion(true);
	model[HEAD]->setExplosion(true);
}

void Player::reviveEnemy()
{
	model[LEGS]->setDesintegration(0);
	model[BODY]->setDesintegration(0);
	model[HEAD]->setDesintegration(0);

    model[LEGS]->setExplosion(false);
	model[BODY]->setExplosion(false);
	model[HEAD]->setExplosion(false);
}

bool Player::isDying()
{
	if (model[LEGS]->isDesintegrating()) return true;
	else return false;
}

void Player::setNick(char *nick)
{
	strcpy(this->nick,nick);
}

char *Player::getNick()
{
	return nick;
}

