#include "Stage.h"

Stage::Stage(char *model, char *mapName)
{
	maxPlayers=0;
	actualPlayers=0;
	
	thirdperson = false;
	camaraObj = new Camera(model);
	enemigos = (Enemy**)malloc(1);
	mensajes = new Message(30.0f);
	mapa = new Map();
	numItemModels = 0;
	numWeapons = 0;
	viewMap = false;
	lights = new Light();
	lights->enable();

	if ( mapName!=NULL )
	if ( loadMap(mapName) == false )
	{
		printf("[E] ERROR LOADING MAP '%s'\n",mapName);
		killProgram(0);
	}
}

void Stage::beginFog()
{
	mapa->beginFog( this->getCamaraPos() );
}

void Stage::endFog()
{
	mapa->endFog();
}

void Stage::setWalkEffect(float f)
{
	step=f;
}

float Stage::getWalkEffect()
{
	return step;
}

void Stage::swapViewMap()
{
	viewMap^=1;
}

bool Stage::getViewMap()
{
	return viewMap;
}

Stage::~Stage()
{
	int i;

	delete mensajes;
	delete lights;
#if _PVS
	delete[] PVS;
#endif
	for (i=0;i<numItems;i++)
		delete itemObj[i];

	delete[] itemObj;

	for (i=0;i<numItemModels;i++)
		delete itemModel[i];

	delete[] itemModel;

	delete camaraObj;
	for (i=0;i<maxPlayers;i++)
		if (enemigos[i]) delete enemigos[i];

	free (enemigos);

	for (i=0;i<numWeapons;i++)
		delete weaponObj[i];

	delete[] weaponObj;

	delete mapa;

	printf("[<] Stage\n");
}

bool Stage::loadMap(char *file)
{
	int contaItems=0;
	FILE *fd;
	char buf[1024];
	int tempoInt=0,tempoInt2=0,numHab,numPortal=0,i=0,cont=0;
	int numFaces,numVertex,numTextures;
	GL3f tempoGL3f;
	GL4i tempoGL4i;
	GL2f tempoGL2f;
	GL3i tempoGL3i;

	float *aVertex1;
	float *aVertex2;
	float *aTextVertex1;
	float *aTextVertex2;
	int *aFaces1;
	int *aFaces2;
	int *aTextFaces1;
	int *aTextFaces2;
	int *aVertexPtr1;
	int *aVertexPtr2;
	
	/* open map */
	chdir(getDirs(file));
	fd=fopen (file,"r");

	if (!fd) return 0;

	
	//Empezamo a leer el fd
	printf("[>] Loading map '%s'\n",file);

	//Creamos el array de habitaciones
	fscanf(fd,"%s",buf);
	fscanf(fd,"%d",&tempoInt);
	mapa->createRooms(tempoInt);

	//Creamos el array de items
	fscanf(fd,"%s",buf);
	fscanf(fd,"%d",&numItems);
	itemObj = new Properties*[numItems];
	printf(" |- Items: %d\n",numItems);

	//Creamos el array de armas
	fscanf(fd,"%s",buf);
	fscanf(fd,"%d",&numWeapons);

	//Le sumo uno teniendo en cuenta la arma por defecto
	numWeapons++;
	if (numWeapons<=0 || numWeapons >10)
	{
		printf("Invalid number of weapons\n");
		killProgram(0);
	}
	weaponObj = new Weapon*[numWeapons];
	printf(" |- Weapons: %d\n",numWeapons);

	//Arma por defecto
	weaponObj[0]=new Weapon("pistola.wph");
	camaraObj->setWeapon(weaponObj[0],weaponObj[0]->getMaxAmmo());

	fscanf(fd,"%s",buf);
	fscanf(fd,"%d",&numItemModels);
	itemModel = new Object3D*[numItemModels];
	printf(" |- ItemsModels: %d\n",numItemModels);
	for (i=0;i<numItemModels;i++)
	{
		fscanf(fd,"%s",buf);
		fscanf(fd,"%d",&tempoInt);
		fscanf(fd,"%s",buf);
		sprintf(buf,"%s.mdh",buf);
		itemModel[tempoInt]=new Object3D(buf);
		printf("  %d:%s\n",tempoInt,buf);
	
	}

	for (i=1;i<numWeapons;i++)
	{
		fscanf(fd,"%s",buf);
		fscanf(fd,"%d",&tempoInt);
		fscanf(fd,"%s",buf);
		weaponObj[i]=new Weapon(buf);
		printf("WEAPON %d: %s ADDED\n",tempoInt,buf);
	}

	//Creamos el array de texturas
	fscanf(fd,"%s",buf);
	numTextures=0;
	if (!strcmp(buf,"NUM_TEXTURES"))
	{
		fscanf(fd,"%d",&numTextures);
		mapa->createTextures(numTextures);
		printf(" |- Textures: %d\n",numTextures);

		// Leemos todas las texturas y las aadimos
		// a un array dentro del objeto mapa
		for (i=0;i<mapa->getNumTextures();i++)
		{
		fscanf(fd,"%s",buf);
		fscanf(fd,"%d",&tempoInt);
		fscanf(fd,"%s",buf);
		mapa->addTexture(tempoInt,buf);
		printf(" |  |- %d: %s\n",tempoInt,buf);
		}
	fscanf(fd,"%s",buf);
	}

	if (strcmp("NUM_STARTS",buf))
	{
		if (!strcmp("MATERIAL",buf))
		{
		float r,g,b,a;

		while(!strcmp("MATERIAL",buf))
		{
			int n;
			fscanf(fd,"%d",&n);
			fscanf(fd,"%s",buf);
			if (!strcmp(buf,"AMB"))
			{
				fscanf(fd,"%f %f %f %f",&r,&g,&b,&a);
				mapa->setMaterial(n,GL_AMBIENT,r,g,b,a);
			} else
			if (!strcmp(buf,"DIF"))
			{
				fscanf(fd,"%f %f %f %f",&r,&g,&b,&a);
				mapa->setMaterial(n,GL_AMBIENT,r,g,b,a);
			} else
			if (!strcmp(buf,"SPE"))
			{
				fscanf(fd,"%f %f %f %f",&r,&g,&b,&a);
				mapa->setMaterial(n,GL_AMBIENT,r,g,b,a);
			} else
			if (!strcmp(buf,"EMI"))
			{
				fscanf(fd,"%f %f %f %f",&r,&g,&b,&a);
				mapa->setMaterial(n,GL_AMBIENT,r,g,b,a);
			} else
			if (!strcmp(buf,"SHI"))
			{
				fscanf(fd,"%f",&r);
				mapa->setMaterial(n,GL_SHININESS,r);
			} else
			if (!strcmp(buf,"AMBDIF"))
			{
				fscanf(fd,"%f %f %f %f",&r,&g,&b,&a);
				mapa->setMaterial(n,GL_AMBIENT_AND_DIFFUSE,r,g,b,a);
			}
			fscanf(fd,"%s",buf);
		}

		} else {
			printf("Invalid MPH : expected : MATERIAL NUM_STARTS\n");
		}
	}

	fscanf(fd,"%d",&tempoInt);

	/* NUM_STARTS */
	/* skipped by client */
	for (i=0;i<tempoInt;i++)
	{
		for (tempoInt2=0;tempoInt2<6;tempoInt2++)
			fscanf(fd,"%s",buf);
	}
		
	/* PVS */
	fscanf(fd,"%s",buf);
	if (!strcmp(buf,"PVS"))
	{
		fscanf(fd,"%s",buf);
#if _PVS
		PVS = new int[mapa->getNumRooms()*mapa->getNumRooms()];
		if(!cargaPVS(buf)) return 0;
		printf("[i] PVS: %s (%dx%d)\n",buf,mapa->getNumRooms(),mapa->getNumRooms());
#endif
		fscanf(fd,"%s",buf);
	}

	if (!strcmp(buf,"AUDIO"))
	{
	fscanf(fd,"%s",buf);
	if(!mapa->loadSound(buf)) return 0;
	printf(" |- Sound '%s'\n",buf);
fscanf(fd,"%s",buf);
	}
	

	if (!strcmp(buf,"RATIO"))
	{
	fscanf(fd,"%f",&RATIO);
	printf(" |- Ratio: %f\n",RATIO);

	fscanf(fd,"%s",buf);
	}

	/* SKYFOG */
	if (!strcmp("SKYFOG",buf))
	{
		int   FogMode;
		int   FogR, FogG, FogB, FogA;
		float FogStart, FogEnd, FogRange, FogDensity;
		float FogX, FogY, FogZ;

		fscanf(fd,"%d %f %f %f %d %d %d %d",
			&FogMode,&FogStart,&FogEnd,&FogDensity,
			&FogR,&FogG,&FogB,&FogA);
		mapa->setSkyFog(
			FogMode,FogStart,FogEnd,FogDensity,
			FogR,FogG,FogB,FogA);
		printf("[i] SKY FOG: %d\n",FogMode);

		/* OTHER FOGS */
		fscanf(fd,"%s",buf);

		while(!strcmp("FOG",buf))
		{
		fscanf(fd,"%d %f %f %d %d %d %d %f %f %f",
			&FogMode,&FogRange,&FogDensity,
			&FogR,&FogG,&FogB,&FogA,
			&FogX,&FogY,&FogZ);
		fscanf(fd,"%s",buf);

		printf(" |- Fog: %d\n", FogMode);

		mapa->addFog(
			FogMode,FogRange,FogDensity,
			FogR,FogG,FogB,FogA,
			FogX,FogY,FogZ);
		}
	}
	
	if (!strcmp("WALLS",buf))
	{
		int i,walls;
		float 	x1,x2,x3,
			y1,y2,y3,
			z1,z2,z3;

		fscanf(fd,"%d",&walls);

		printf("[i] Collisions: %d\n",walls);

		for (i=0;i<walls;i++)
		{
			fscanf(fd,"%f %f %f",&x1,&y1,&z1);
			fscanf(fd,"%f %f %f",&x2,&y2,&z2);
			fscanf(fd,"%f %f %f",&x3,&y3,&z3);
// XXX must use RATIO!
#if 0
			mapa->addCollision(
				x1*RATIO,y1*RATIO,z1*RATIO,
				x2*RATIO,y2*RATIO,z2*RATIO,
				x3*RATIO,y3*RATIO,z3*RATIO );
#else
			mapa->addCollision(
				x1,y1,z1,
				x2,y2,z2,
				x3,y3,z3 );
#endif
		}
	}

	// HABITACION
	fscanf(fd,"%s",buf);

	if (!strcmp("LIGHTS",buf))
	{
		int number;
		fscanf(fd,"%d", &number);
		printf("[i] Lights: %d\n",number);

		fscanf(fd,"%s", buf);
		if (!strcmp(buf,"AMBIENT"))
		{
			float r,g,b,a;
			fscanf(fd,"%f %f %f %f",&r,&g,&b,&a);
			lights->setAmbient(r,g,b,a);
			fscanf(fd,"%s",buf);
		}
		int lid;

		if (strcmp(buf,"--"))
		{
		printf("invalid keyword '%s' after LIGHTS\n",buf);
		}

		fscanf(fd,"%s",buf);

		bool spot=false;
		float pos[3];

		for (int i=0;i<number;i++)
		{
			lid=lights->add();
		  do
		  {
			if (!strcmp(buf,"POS"))
			{
				float x,y,z;
				fscanf(fd,"%f %f %f",&x,&y,&z);
				lights->setPosition(lid,x,y,z);
				pos[0]=x;pos[1]=y;pos[2]=z;
			} else
			if (!strcmp(buf,"DIR"))
			{
				float x,y,z;
				fscanf(fd,"%f %f %f",&x,&y,&z);
				lights->setDirection(lid,x,y,z);
				spot=true;
			} else
			if (!strcmp(buf,"AMB"))
			{
				float r,g,b,a;
				fscanf(fd,"%f %f %f %f",&r,&g,&b,&a);
				lights->setAmbient(lid,r,g,b,a);
			}
			if (!strcmp(buf,"DIF"))
			{
				float r, g, b, a;
				fscanf(fd,"%f %f %f %f",&r,&g,&b,&a);
				lights->setDiffuse(lid,r,g,b,a);
			} else
			if (!strcmp(buf,"SPE"))
			{
				float r,g,b,a;
				fscanf(fd,"%f %f %f %f",&r,&g,&b,&a);
				lights->setSpecular(lid,r,g,b,a);
			}
			if (!strcmp(buf,"COF"))
			{
				float cof;
				fscanf(fd,"%f",&cof);
				lights->setCutOff(lid,cof);
			} else
			if (!strcmp(buf,"--"))
			{
				fscanf(fd,"%s",buf);
				break;
			}

			fscanf(fd,"%s",buf);

		  } while(true);

			if (spot)
			printf(" |- Spot: (%f,%f,%f)\n",pos[0],pos[1],pos[2]);
			else
			printf(" |- Omni: (%f,%f,%f)\n",pos[0],pos[1],pos[2]);
		}

	}
	if (strcmp(buf,"HABITACION"))
	{
		printf("ERROR: Expected(HABITACION) Received(%s)\n",buf);
		killProgram(1);
	}

	/* loading rooms */
	bool portals=false;

	printf("[i] Rooms: %d",mapa->getNumRooms());
	for (i=0;i<mapa->getNumRooms();i++)
	{
		numPortal=0;

			
		//Cargamos la habitacion en la clase mapa
		fscanf(fd,"%d",&numHab);
		mapa->addRoom(numHab);

		printf("\n");

		portals=false;
		fscanf(fd,"%s %d",buf,&tempoInt);
		if (strcmp(buf,"NUM_PORTALS"))
		{
			printf("invalid MPH file\n");
		}
#if _PVS		
		if (!mapa->getRoom(numHab)->createPortals(tempoInt))
		{
			printf("ERROR CREATING PORTALS\n");
			return 0;
		}
#endif
		//Buscamos todos los elementos que contiene
		do {
			fscanf(fd,"%s",buf);
			if (!strcmp(buf,"HABITACION")) break;
			fscanf(fd,"%s",buf);

			int numItemUsed=-1;
			//Miramos si encuentra un item
			if (!strcmp(buf,"ITEM"))
			{
				fscanf(fd,"%d",&tempoInt2);
				printf(" |- Items: %d@%d\n",tempoInt2,numHab);
											
				itemObj[contaItems]=new Properties(itemModel[tempoInt2]);
				//Carga el item en el array de items, le asigna el id correspondiente a la
				//habitacion donde se encuentra, coge su posicion y aumenta el contador de items
#if _PVS				
				itemObj[contaItems]->setId(numHab);
#endif
				fscanf(fd,"%f %f %f",&tempoGL3f.x,&tempoGL3f.y,&tempoGL3f.z);
				tempoGL3f=tempoGL3f*RATIO;
				tempoGL3f.y+=0.05f;
				itemObj[contaItems]->setPos(tempoGL3f);
				itemObj[contaItems]->setRotationAngle(ROTATE_ANGLE_ITEMS);
				itemObj[contaItems]->activateRotation(ROTATE_SPEED_ITEMS);

				contaItems++;
			} else
			if (!strcmp(buf,"WEAPON"))
			{
				fscanf(fd,"%d",&tempoInt2);
				
				printf(" | Weapon: %d@%d\n",tempoInt2,numHab);
				itemObj[contaItems]=new Properties(weaponObj[tempoInt2]->getThirdView()->getObject3D());
				//Carga el item en el array de items, le asigna el id correspondiente a la
				//habitacion donde se encuentra, coge su posicion y aumenta el contador de items
#if _PVS				
				itemObj[contaItems]->setId(numHab);
#endif
				fscanf(fd,"%f %f %f",&tempoGL3f.x,&tempoGL3f.y,&tempoGL3f.z);
				tempoGL3f=tempoGL3f*RATIO;
				itemObj[contaItems]->setPos(tempoGL3f);
				itemObj[contaItems]->setRotationAngle(ROTATE_ANGLE_ITEMS);
				itemObj[contaItems]->activateRotation(ROTATE_SPEED_ITEMS);

				contaItems++;
			} else
			//Miramos si encuentra un portal
			if(!strcmp(buf,"PORTAL"))
			{
				GL3f *vertex;
				vertex=new GL3f[4];
#if _PVS
				//Creamos variables temporales
				GLBounding bounding= GLBounding();
		
#endif
				//Le asignamos una habitacion destino al portal
				fscanf(fd,"%d",&tempoInt2);
if (!portals)
{
	printf(" |- Room %d have portals to : %d",numHab,tempoInt2);
	portals=true;
} else {
	printf(",%d",tempoInt2);
}
#if _PVS
				mapa->getRoom(numHab)->setPortalNeighbour(numPortal,tempoInt2);
#endif
				//Leemos sus vertices y creamos un array con ellos
				for (tempoInt2=0;tempoInt2<4;tempoInt2++)
				{
					fscanf(fd,"%f %f %f",&vertex[tempoInt2].x,&vertex[tempoInt2].y,&vertex[tempoInt2].z);
#if _PVS
					vertex[tempoInt2]=vertex[tempoInt2]*RATIO;
#endif
				}
#if _PVS
				mapa->getRoom(numHab)->setPortalVertex(numPortal,vertex);
				delete vertex;
				numPortal++;
#endif
			} else
			//Cargamos la habitacion en si
			if(!strcmp(buf,"HAB"))
			{
				fscanf(fd,"%s",buf);
				fscanf(fd,"%d",&numVertex);

				//Guardamos todos los vertices en un array
				aVertex1=new float[numVertex*3*2];
				for (cont=0;cont<numVertex*3;cont+=3)
				{
					fscanf(fd,"%f %f %f",
						&aVertex1[cont],
						&aVertex1[cont+1],
						&aVertex1[cont+2]);
					aVertex1[cont]*=RATIO;
					aVertex1[cont+1]*=RATIO;
					aVertex1[cont+2]*=RATIO;
				}

				fscanf(fd,"%s",buf);
				fscanf(fd,"%d",&numFaces);

				aFaces1=new int[numFaces*3];
				aFaces2=new int[numFaces*3];
				aVertexPtr1=new int[numFaces*2];

				//Esta variable almacenara el vertexArray.
				//Se multiplica por 9 el numero de caras ya que cada cara tiene 3 vertices y
				//cada uno 3 componentes (x,y,z)
				aVertex2=new float[numFaces*9];

				//Guardamos las caras en un array y la textura que les corresponde en otro
				for (cont=0;cont<numFaces;cont++)
				{
					fscanf(fd,"%d %d %d %d",
						&aFaces1[(cont*3)],
						&aFaces1[(cont*3)+1],
						&aFaces1[(cont*3)+2],
						&aVertexPtr1[cont]);
				}

				fscanf(fd,"%s",buf);
				fscanf(fd,"%d",&numVertex);

				aTextVertex1 = new float[numVertex*2];

				//Guardamos el array de vertices de textura	
				for (cont=0;cont<numVertex*2;cont+=2)
				{
					fscanf(fd,"%f %f",
						&aTextVertex1[cont],
						&aTextVertex1[cont+1]);
				}
				
				fscanf(fd,"%s",buf);
				fscanf(fd,"%d",&numFaces);
				aTextFaces1 = new int[numFaces*3];
				aTextFaces2 = new int[numFaces*3];

				//Esta variable almacenara el vertexArray.de texturas
				//Se multiplica por 6 el numero de caras ya que cada cara tiene 3 vertices y
				//cada uno 2 componentes (u,v)
				aTextVertex2 = new float[numFaces*6];

				//Guardamos el array de caras de textura
				for (cont=0;cont<numFaces*3;cont+=3)
				{
					fscanf(fd,"%d %d %d",
						&aTextFaces1[cont+0],
						&aTextFaces1[cont+1],
						&aTextFaces1[cont+2]);
				}


				//Esta variable se desactivara cuando encontremos una textura nueva
				bool newTexture=true;
				int numHabTextures=0;

				//Buscamos el numero de texturas
				for (cont=0;cont<numTextures;cont++)
				{
					newTexture=true;
					for (int cont2=0;cont2<numFaces && newTexture;cont2++)
					{
						//Miramos en todas las caras hasta encontrar una que tenga la textura
						//que estamos buscando
						if (aVertexPtr1[cont2]==cont)
						{
							//Si le encuentra, aumentamos el contador de texturas y desactivamos
							//el flag para buscar una nueva textura
							numHabTextures++;
							newTexture=false;
						}
					}
				}
							
				//Inicializamos el vertexPointer que almacenara la posicion del array de
				//vertices donde empieza una textura y a que textura se refiere
				aVertexPtr2 = new int[(numHabTextures*2)+1];

				//Variables temporales
				int cont3=0,cont4=0;

				//Llenamos el vertex pointer y ordenamos las caras
				for (cont=0;cont<numTextures;cont++)
				{
					newTexture=true;
					for (int cont2=0;cont2<numFaces;cont2++)
					{
						if (aVertexPtr1[cont2]==cont)
						{
							if (newTexture)
							{
								aVertexPtr2[cont3]=cont4;
								aVertexPtr2[cont3+1]=cont;
								newTexture=false;
								// printf("Inicio %d Textura %d\n",aVertexPtr2[cont3],aVertexPtr2[cont3+1]);

								cont3+=2;
							}
							aFaces2[cont4]=aFaces1[(cont2*3)];
							aFaces2[cont4+1]=aFaces1[(cont2*3)+1];
							aFaces2[cont4+2]=aFaces1[(cont2*3)+2];
							aTextFaces2[cont4]=aTextFaces1[(cont2*3)];
							aTextFaces2[cont4+1]=aTextFaces1[(cont2*3)+1];
							aTextFaces2[cont4+2]=aTextFaces1[(cont2*3)+2];
							
							cont4+=3;
						} else
						if (aVertexPtr1[cont2]>=numTextures) 
							printf ("ERROR (Texture out of range) Pointer= %d NumTextures= %d\n",aVertexPtr1[cont2],numTextures);
					}
				}

				//En la ultima posicion del vertex poineter guardo la longitud del array para
				//saber cuando acaba este

				aVertexPtr2[cont3]=numFaces*3;

				//Ordenamos los vertices
				for (cont=0;cont<numFaces*3;cont++)
				{
					aVertex2[(cont*3)]=aVertex1[aFaces2[cont]*3];
					aVertex2[(cont*3)+1]=aVertex1[(aFaces2[cont]*3)+1];
					aVertex2[(cont*3)+2]=aVertex1[(aFaces2[cont]*3)+2];

					aTextVertex2[(cont*2)]=aTextVertex1[(aTextFaces2[cont]*2)];
					aTextVertex2[(cont*2)+1]=aTextVertex1[(aTextFaces2[cont]*2)+1];
				}

				//Le pasamos toda la informacion a la habitacion correspondiente
				mapa->getRoom(numHab)->addVertex(aVertex2,numFaces);
				mapa->getRoom(numHab)->addTextureVertex(aTextVertex2,numFaces);
				mapa->getRoom(numHab)->addTexturePointer(aVertexPtr2,numHabTextures);

				delete[] aVertex1;
				delete[] aVertex2;
				delete[] aTextVertex1;
				delete[] aTextVertex2;
				delete[] aFaces1;
				delete[] aFaces2;
				delete[] aTextFaces1;
			    	delete[] aTextFaces2;
				delete[] aVertexPtr1;
				delete[] aVertexPtr2;

			}

		}while(!feof(fd));
	}

	//Cerramos el fd
	fclose(fd);

	printf("\n[i] SkyBox: '%s'\n","sky1.skh");
	if ( !mapa->loadSky("sky1.skh") )
	{
		printf("Couldn't load sky1.skh or one of his components\n");	
		return 0;
	}

	return 1;
}

void Stage::update()
{
	int j;

	/* update items */
	for (j=0;j<numItems;j++)
		itemObj[j]->update();

	/* update enemies */
	for (j=0;j<maxPlayers;j++)
		if (enemigos[j]) enemigos[j]->update();

	/* update camera */
	camaraObj->update();

	/* update lights */
	lights->update( getCamaraPos(), getRotation() );
}

#if _PVS
int Stage::cargaPVS(char *file)
{
	FILE* fd;
	
	chdir(getDirs(file));
	fd = fopen (file,"r");

	if(!fd) return 0;

	int tempoInt,x,y;

	//Cojemos el numero de habitaciones del mapa
	int numHabitaciones = mapa->getNumRooms();

	//Nos saltamos la primera linia ya que no tiene informacion interesante
	for (x=0;x<=numHabitaciones;x++)
		fscanf(fd,"%d",&tempoInt);
	
	//Vamos leyendo el archivo y llenando el PVS
	for(y=0;y<numHabitaciones;y++)
	{
		//El primer numero no sirve de nada y lo pasamos
		fscanf(fd,"%d",&tempoInt);
		for (x=0;x<numHabitaciones;x++)
		{
			fscanf(fd,"%d",&tempoInt);
			if (tempoInt>1 || tempoInt<0) return 0;
			PVS[(y*numHabitaciones)+x]=tempoInt;
		}
	}

	return 1;
}
#endif

Properties *Stage::getItem(int pos)
{
	if (pos>=0 && pos<=numItems) return itemObj[pos];
	else return 0;
}

#if _PVS
int *Stage::getPVS(int pos)
{
	int *PVS_tempo,j=0;
	int numHabitaciones=mapa->getNumRooms();

	//Creamos un array en el cual guardamos el PVS de la habitacion que le pidamos
	PVS_tempo=new int[numHabitaciones];
	for (int i=pos*numHabitaciones;i<numHabitaciones+(pos*numHabitaciones);i++,j++){
		PVS_tempo[j]=PVS[i];
	}
	return PVS_tempo;
}
#endif

void Stage::setThirdPerson(bool var)
{
	thirdperson=var;
}

void Stage::setCamaraPos(GL3f posCam,bool collision)
{
	//Antes de pasarle la nueva posicion a la camara, comprobamos que no cambiamos de habitacion
#if _PVS
	if (collision)
	{
		int id=checkCamera(posCam);
		if (id!=camaraObj->getId())	printf ("Actual Hab: %d\n",id);
		if (id>=0 && id<mapa->getNumRooms()) camaraObj->setId(id);
	}
#endif

	camaraObj->setPos(posCam);
}

Player* Stage::getMyBody()
{
	return camaraObj;
}

void Stage::setCamaraObj(double yawH, double yawV)
{
	if (thirdperson)
		yawV=0;

	camaraObj->setObj(yawH, yawV);
}

float Stage::getRotation()
{
	return camaraObj->getRotation();
}
GL2f Stage::getCamaraYaw()
{
	GL2f yaw=camaraObj->getYaw();
	if (thirdperson)
		yaw.y=0;
	return yaw;
}

GLBounding Stage::getCamaraBounding()
{
	return camaraObj->getBounding();
}

#if _PVS
int Stage::checkCamera(GL3f pos)
{
	Room *habTempo=mapa->getRoom(camaraObj->getId());

	GL3f intersectionPoint;
	GL3f linia[]={camaraObj->getPos(), pos};

	GL3f poli1[3];
	double orig[3];
	double dir[3];

	GL3f tempo;
	tempo=linia[1] - linia [0];

	for (int i=0;i<habTempo->getNumPortals();i++){
		GL3f *vertex_=habTempo->getPortalVertex(i);
		
		
		// CUTRADA
		poli1[0] = vertex_[0];
		poli1[1] = vertex_[1];
		poli1[2] = vertex_[3];

		orig[0] = linia[0].x;
		orig[1] = linia[0].y;
		orig[2] = linia[0].z;

		dir[0] = (linia[1].x - linia[0].x);
		dir[1] = (linia[1].y - linia[0].y);
		dir[2] = (linia[1].z - linia[0].z);

		
		double t = testline(linia, intersectionPoint, poli1);
		if ( (t>0.0) && (t <= 1.0) ) {
		/*	printf("portal:\n");
			poli1[0].print();
			poli1[1].print();
			poli1[2].print();
			printf("linia: \n");
			linia[0].print();
			linia[1].print();
			printf("\n");
*/
	//		printf("COLISION CON EL PLANO\n");
			if ( InsidePolygon(intersectionPoint, poli1,3)) {
				return habTempo->getPortalNeighbour(i);
			} else {
				poli1[0] = vertex_[0];
				poli1[1] = vertex_[2];
				poli1[2] = vertex_[3];

				if ( InsidePolygon(intersectionPoint, poli1,3)) {
					return habTempo->getPortalNeighbour(i);
				}
			}
		}
	
		
		
	}
	
	return camaraObj->getId();
}
#endif

int Stage::getNumItems()
{
	return numItems;
};

int Stage::getCamaraId()
{
	return camaraObj->getId();
}

GL3f Stage::getCamaraPos()
{
	GL3f pos=camaraObj->getPos();
	
	return pos;
}

GL3f Stage::getCamaraObj()
{
	return camaraObj->getObjPos();
}

Message *Stage::getMessageControl()
{
	return mensajes;
}

Map *Stage::getMap()
{
	return mapa;
}

bool Stage::addEnemy(char *ruta, char *nick, int pos)
{
	if (pos>=maxPlayers)
	{
		enemigos=(Enemy**)realloc(enemigos,sizeof(Enemy**)*(pos+1));
		if (!enemigos) exit(0);
		for (int i=maxPlayers;i<pos+1;i++) 
			enemigos[i]=0;
		maxPlayers=pos+1;
	}

	enemigos[pos]=new Enemy(ruta);
	enemigos[pos]->setNick(nick);
	enemigos[pos]->setWeapon(weaponObj[0],weaponObj[0]->getMaxAmmo());
	actualPlayers++;

	return true;
}

int Stage::getMaxPlayers()
{
	return maxPlayers;
}

Enemy *Stage::getEnemy(int which)
{
	if (which>=0 && which<maxPlayers){
		return enemigos[which];
	}
	return 0;
}

bool Stage::killEnemy(int who)
{
	if (enemigos[who]) delete enemigos[who];
	else return false;
	enemigos[who]=0;
	actualPlayers--;
	if (who+1==maxPlayers) {
		maxPlayers--;
		enemigos=(Enemy**)realloc(enemigos,sizeof(Enemy**)*who+1);
	}
	return true;
}

int Stage::getActualPlayers()
{
	return actualPlayers;
}

Properties *Stage::getCamFirstView()
{
	return camaraObj->getWeapon()->getFirstView();
}

Properties *Stage::getCamThirdView()
{
	return camaraObj->getWeapon()->getThirdView();
}

void Stage::setCamaraDNI(int num)
{
	camaraObj->setDNI(num);
}

int Stage::getCamaraDNI()
{
	return camaraObj->getDNI();
}

void Stage::setCamaraNick(char *nick)
{
	camaraObj->setNick(nick);
}

char *Stage::getCamaraNick()
{
	return camaraObj->getNick();
}

void Stage::setCamaraId(int id)
{
#if _PVS
	camaraObj->setId(id);
#endif
}

bool Stage::existsEnemy(int who)
{
	if (who<0 || who>=maxPlayers) return false;
	if (enemigos[who]) return true;
	else return false;
}

Weapon *Stage::getCamaraWeapon()
{
	return camaraObj->getWeapon();
}

Interface *Stage::getCamaraInterface()
{
	return camaraObj->getInterface();
}

bool Stage::isThirdView()
{
	return thirdperson;
}
