#include "TGAloader.h"
#include <GL/glu.h>


unsigned char *TGAloader(char *file,int *w, int *h, int *bpp)
{
	FILE *fd=0;
	unsigned char *filebuf;
	unsigned char *tex;
	long fsize=0;
	long i;
	int size,color;

	//printf("_________________ %s\n",file);
	
	#if BZIP2
	char a[BZ_MAX_UNUSED];
	int bzerr=0,nread=0;
	char obuf[1024];
	BZFILE *bzf=0;
	#endif
	
	try {
		chdir(getDirs(file)); 
	} catch (int e) {
		printf("CANNOT CHDIR!!! XXX\n");
		return 0;
	}
		
#if HAVE_SDL_IMAGE
	SDL_Surface *img = IMG_Load(file);
	if (!img)
	{
		printf("Cannot open texture '%s'\n",file);
		return 0;
	}
	*w=img->w;
	*h=img->h;
	*bpp=img->format->BytesPerPixel*8;
	SDL_Surface *image = SDL_CreateRGBSurface(SDL_HWSURFACE|SDL_ASYNCBLIT,
		img->w,img->h,*bpp,
		#if SDL_BYTEORDER == SDL_LIL_ENDIAN /* OpenGL RGBA masks */
		0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000
		#else
		0xFF000000, 0x00FF0000, 0x0000FF00, 0x000000FF
		#endif
		);
	if (!image)
	{
		printf("Failed to create SDL_Surface\n");
		return 0;
	}
	//printf("%s: w(%d) h(%d) bpp(%d)\n",file,*w,*h,*bpp);
	SDL_SetAlpha(img, 0, 0);
	SDL_BlitSurface(img, NULL, image, NULL);

	// image->pixels -> data
	fsize=(*w)*(*h)*(*bpp/8);
	tex=(unsigned char *)malloc(fsize);
	memcpy(tex,image->pixels,fsize);

	// texture loaded ok
	SDL_FreeSurface(image);
	SDL_FreeSurface(img);
#else
	fd=fopen(file,"rb");

	if (!fd) 
	{
	  printf("Cannot open texture '%s'\n",file);
	  return 0;
	};

	// is bzip2 tga?
	#if BZIP2
	bzf=BZ2_bzReadOpen(&bzerr,f,0,0,a,0);
	if (!bzf)
		printf("SUXED\n");
	filebuf=(char *)malloc(1);
	while(!bzerr)
	{
		nread=BZ2_bzRead(&bzerr,bzf,obuf,1024);

		if ((!bzerr||bzerr==BZ_STREAM_END) && nread) {
			filebuf=(char *)realloc(filebuf,fsize+nread);
			memcpy(filebuf+fsize,obuf,nread);
			fsize+=nread;
			}
		} 
	if (bzerr&&bzerr!=BZ_STREAM_END) { //isn't bzip2
	#endif
	//printf("ISNT BZ2:%s\n",file);
	fseek (fd, 0,SEEK_END); // Move to the end
	fsize=ftell(fd);      // Get file size
	fseek(fd,0,SEEK_SET); // Move to the beggining
	filebuf=(unsigned char *)malloc(fsize);
	fread(filebuf,fsize,1,fd); // read file to buffer.

	#if BZIP2
	}
	#else
	if (	filebuf[0]=='B' && /* Is BzLib and we don't have bzlib */
		filebuf[1]=='Z' ) {
		return 0;
	}
	#endif
	// Check Header
	i=TGAHEAD; // header size
	//while((--i==2)?--i:i)(filebuf[i])?throw 1:i;
	*w=(filebuf[TGAHEAD+1]<<8)+filebuf[TGAHEAD+0];
	*h=(filebuf[TGAHEAD+3]<<8)+filebuf[TGAHEAD+2];

	if (*w<0) (*w)*=-1;
	if (*h<0) (*h)*=-1;

	*bpp=filebuf[TGAHEAD+4];
	if (*bpp==16)
	{
		printf("invalid BPP 8\n");
		return 0;
	}

	if (w&&h)	size=(*w)*(*h)*(*bpp/8);
	else return 0; 		// throw if null image

	tex=(unsigned char *)malloc(fsize);
	switch(*bpp)
	{
	case 24:
		if ((*w)*(*h)*(*bpp/8)>fsize) return 0;
		break;
	case 32:
		break;
	}

	for(i=0;i<size;i+=(*bpp/8)) // BGR 2 RGB and BPP conversion
	{
		switch(*bpp)
		{
		case 8:
			// unsupported
			break;
		case 16:
			color=filebuf[TGAMAP16+i]+(filebuf[TGAMAP16+i+1]<<8);
			tex[i+0]=(color>>7) & 0xf8;
			tex[i+1]=(color>>2) & 0xf8;
			tex[i+2]=(color<<3);
			break;
		case 24:
			tex[i+0]=filebuf[TGAMAP24+i+2];
			tex[i+1]=filebuf[TGAMAP24+i+1];
			tex[i+2]=filebuf[TGAMAP24+i+0];
			break;
		case 32:
			tex[i+0]=filebuf[TGAMAP32+i+3];
			tex[i+1]=filebuf[TGAMAP32+i+2];
			tex[i+2]=filebuf[TGAMAP32+i+1];
			tex[i+3]=filebuf[TGAMAP32+i+0];
			break;
		}
	}
	free (filebuf);      // Alliberem el buffer.

	fclose(fd);

#endif
	return tex;
}

unsigned int TGAloader_tid(char *file)
{
	int bpp;
	return TGAloader_tid(file,&bpp);
}

unsigned int TGAloader_tid(char *file,int *bpp)
{
	int w,h;
	unsigned int tid;
	unsigned char *ptr;

        ptr=TGAloader(file,&w,&h,bpp);
	if (ptr==0) return 0;

        glGenTextures(1,&tid);
        glBindTexture(GL_TEXTURE_2D,tid);

        /* set texture parameters */
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);

        /* load image into vRAM */
        if (*bpp==32)
                gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,w,h,GL_RGBA,GL_UNSIGNED_BYTE,ptr);
        else
                gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,w,h,GL_RGB,GL_UNSIGNED_BYTE,ptr);
                
        /* Free RAM image */
        free(ptr);

	return tid;
}
