#include "main.h"

void show_usage(char *argv0)
{
printf(	"glH v%s - usage '%s' [flags]\n"
	" -d [datadir]   - change default data dir (%s)\n"
	" -m [mapfile]   - set map to be loaded (.mph).\n"
	" -c [carfile]   - set character file (.car) to be used.\n"
	" -n             - disable network (for testing)\n"
	" -l             - disable lights\n"
	" -s             - disable sound\n"
	" -f             - fullscreen\n"
	" -v             - verbose\n"
	" -h             - show this helpy stuff\n",VERSION,argv0,GLHDATADIR);
exit(0);
}


int
main(int argc, char **argv)
{
	FILE *fd;
	char buf[80];
	bool skip_menu=false;
	char *model=0;
	char *map=0;
	game=0;
	
	chCfgDir(); // move to config directory

#if !WIN32
	int c;
	while((c=getopt(argc,argv,"d:vhnm:lc:sf")) != -1)
	{
		switch(c)
		{
		case 'f':
			FULLSCREEN=1;
			break;
		case 'd':
			setDataDir(optarg);
			break;
		case 'v':
			DEBUG=true;
			break;
		case 'h':
			show_usage(argv[0]);
			break;
		case 'm':
			map=optarg;
			break;
		case 'c':
			model=optarg;
			break;
		case 's':
			SOUND_ENABLED=0;
			break;
		case 'l':
			LIGHT=0;
			break;
		case 'n':
			skip_menu=true;
			NETWORK=0;
			break;
		}
	}
#endif
	if (map==NULL) map=strdup("castle.mph");

	if (checkDataDir()==false)
	{
		printf("[ERROR] No data directory found.\n");
		killProgram(1);
	}

	/* init graphics */
	new SDL();

	/* ## MAIN MENU ## */
__main_menu:
	if (! skip_menu)
	{
		MainMenu *mm=new MainMenu();

		do 
		{
			renderDelay=SDL_GetTicks()-lastRender;
			lastRender=SDL_GetTicks();

			mm->drawScene();
			SDL_GL_SwapBuffers( );

			mm->getKeys();

		} while( !mm->isFinished() );

		NETWORK=1;
		switch( mm->getOption() )
		{
			case MENU_EXIT:
				killProgram(0);
				break;
			case MENU_CLIENT:
			case MENU_SERVER:
			case MENU_OPTIONS:
				printf("Not yet supported\n");
				killProgram(0);
				break;
			case MENU_OFFLINE:
				NETWORK=0;
				break;
		}
		delete mm;
	}

	if (!model)
	{
	/* Choose person */
	ChooseBody *cb=new ChooseBody();
	do {

		cb->drawScene();
		cb->getKeys();

		SDL_GL_SwapBuffers( );

	} while (!cb->isFinished());

	switch(cb->getOption())
	{
	case BODY_EXIT:
		model=0;
		delete cb;
		if (skip_menu)
			killProgram(0);
		goto __main_menu;
		break;
	}
	model=cb->getModel();
	delete cb;
	}

	/* ## GO GO ! ## */

	// We create the object that will control the game
	game=new Game(model,map);

	// The main game loop :)
	for (;;)
	{
		//Let's paint it all
		game->draw();

		//Here we swap the buffers
		SDL_GL_SwapBuffers( );
		
		renderDelay=SDL_GetTicks()-lastRender;
		lastRender=SDL_GetTicks();

		//Let's see if any key is pressed
		game->input();
	}

return 1;
} // main

#if WIN32
int WINAPI WinMain (HINSTANCE hInstance, 
                        HINSTANCE hPrevInstance, 
                        LPSTR szCmdLine, 
                        int iCmdShow) 
{
	return main(0,0);
}
#endif

void killProgram(int mode)
{
	if (game);
	delete game;

	SDL_Quit();

	printf("\n[i] Termination: %s\n",(mode)?"ERROR":"OK");

	// Show Credits
	printf("\nglH v%s IS DOWN\n\n"
	       "******* GLH TEAM **************************************************************\n"
	       "*                                                                             *\n"
	       "*  BASE 3D ENGINE: Ankh (David Barrachina Martinez)                           *\n"
	       "*  AUDIO,NETWORK,ENGINE,TOOLS: Pancake (Sergi Alvarez Capilla)                *\n"
	       "*  DESIGN AND 3D MODELS: KonstaN (Marc Vidal Xarto)                           *\n"
	       "*  SUPERVISOR: OpenGL teachers: Vicenc Garcia, Oscar Garcia.                  *\n"
	       "*                                                                             *\n"
	       "*******************************************************************************\n"
	       "* This is Free Software. You can use, modify and redistribute this package as *\n"
	       "* GPLv2 says. It means that source modifications must be free and opensourced *\n"
	       "*******************************************************************************\n"
	       "*                                                                             *\n"
	       "*                  CHECK FOR NEW UPDATES IN: http://glh.sf.net/               *\n"
	       "*                                                                             *\n"
	       "*******************************************************************************\n",VERSION);

	exit(mode);
}



